from django.db import migrations


def update_objecttype_features(apps, schema_editor):
    """Refresh the ObjectType features for RIRConfig to include 'jobs'."""
    ObjectType = apps.get_model("core", "ObjectType")
    try:
        ot = ObjectType.objects.get(app_label="netbox_rir_manager", model="rirconfig")
        if "jobs" not in ot.features:
            ot.features.append("jobs")
            ot.save(update_fields=["features"])
    except ObjectType.DoesNotExist:
        pass


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0018_concrete_objecttype"),
        ("netbox_rir_manager", "0007_alter_rirconfig_api_url"),
    ]

    operations = [
        migrations.RunPython(update_objecttype_features, migrations.RunPython.noop),
    ]



# Generated by Django 5.2.9 on 2026-02-05 01:17

import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("extras", "0134_owner"),
        ("netbox_rir_manager", "0002_rename_riraccount_to_rirconfig"),
        ("tenancy", "0022_add_comments_to_organizationalmodel"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name="rircontact",
            name="contact",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="rir_contacts",
                to="tenancy.contact",
            ),
        ),
        migrations.CreateModel(
            name="RIRUserKey",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("api_key", models.CharField(max_length=255)),
                (
                    "rir_config",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="user_keys",
                        to="netbox_rir_manager.rirconfig",
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="rir_user_keys",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["user", "rir_config"],
                "constraints": [models.UniqueConstraint(fields=("user", "rir_config"), name="unique_user_rir_config")],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
    ]



from unittest.mock import MagicMock, patch

import pytest
from django.urls import reverse


@pytest.mark.django_db
class TestRIRConfigViews:
    def test_list_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:rirconfig_list")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_detail_view(self, admin_client, rir_config):
        url = reverse("plugins:netbox_rir_manager:rirconfig", args=[rir_config.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_add_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:rirconfig_add")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_edit_view(self, admin_client, rir_config):
        url = reverse("plugins:netbox_rir_manager:rirconfig_edit", args=[rir_config.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_delete_view(self, admin_client, rir_config):
        url = reverse("plugins:netbox_rir_manager:rirconfig_delete", args=[rir_config.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_changelog_view(self, admin_client, rir_config):
        url = reverse("plugins:netbox_rir_manager:rirconfig_changelog", args=[rir_config.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_list_view_with_data(self, admin_client, rir_config):
        url = reverse("plugins:netbox_rir_manager:rirconfig_list")
        response = admin_client.get(url)
        assert response.status_code == 200
        assert rir_config.name.encode() in response.content


@pytest.mark.django_db
class TestRIROrganizationViews:
    def test_list_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:rirorganization_list")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_detail_view(self, admin_client, rir_organization):
        url = reverse("plugins:netbox_rir_manager:rirorganization", args=[rir_organization.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_edit_view(self, admin_client, rir_organization):
        url = reverse("plugins:netbox_rir_manager:rirorganization_edit", args=[rir_organization.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_delete_view(self, admin_client, rir_organization):
        url = reverse("plugins:netbox_rir_manager:rirorganization_delete", args=[rir_organization.pk])
        response = admin_client.get(url)
        assert response.status_code == 200


@pytest.mark.django_db
class TestRIRContactViews:
    def test_list_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:rircontact_list")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_detail_view(self, admin_client, rir_contact):
        url = reverse("plugins:netbox_rir_manager:rircontact", args=[rir_contact.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_edit_view(self, admin_client, rir_contact):
        url = reverse("plugins:netbox_rir_manager:rircontact_edit", args=[rir_contact.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_delete_view(self, admin_client, rir_contact):
        url = reverse("plugins:netbox_rir_manager:rircontact_delete", args=[rir_contact.pk])
        response = admin_client.get(url)
        assert response.status_code == 200


@pytest.mark.django_db
class TestRIRNetworkViews:
    def test_list_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:rirnetwork_list")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_detail_view(self, admin_client, rir_network):
        url = reverse("plugins:netbox_rir_manager:rirnetwork", args=[rir_network.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_edit_view(self, admin_client, rir_network):
        url = reverse("plugins:netbox_rir_manager:rirnetwork_edit", args=[rir_network.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_delete_view(self, admin_client, rir_network):
        url = reverse("plugins:netbox_rir_manager:rirnetwork_delete", args=[rir_network.pk])
        response = admin_client.get(url)
        assert response.status_code == 200


@pytest.mark.django_db
class TestRIRSyncLogViews:
    def test_list_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:rirsynclog_list")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_detail_view(self, admin_client, rir_config):
        from netbox_rir_manager.models import RIRSyncLog

        log = RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="TEST", status="success"
        )
        url = reverse("plugins:netbox_rir_manager:rirsynclog", args=[log.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_delete_view(self, admin_client, rir_config):
        from netbox_rir_manager.models import RIRSyncLog

        log = RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="TEST", status="success"
        )
        url = reverse("plugins:netbox_rir_manager:rirsynclog_delete", args=[log.pk])
        response = admin_client.get(url)
        assert response.status_code == 200


@pytest.mark.django_db
class TestRIRUserKeyViews:
    def test_list_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:riruserkey_list")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_detail_view(self, admin_client, rir_user_key):
        url = reverse("plugins:netbox_rir_manager:riruserkey", args=[rir_user_key.pk])
        response = admin_client.get(url)
        assert response.status_code == 200


@pytest.mark.django_db
class TestRIRTicketViews:
    def test_list_view(self, admin_client):
        url = reverse("plugins:netbox_rir_manager:rirticket_list")
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_detail_view(self, admin_client, rir_ticket):
        url = reverse("plugins:netbox_rir_manager:rirticket", args=[rir_ticket.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_delete_view(self, admin_client, rir_ticket):
        url = reverse("plugins:netbox_rir_manager:rirticket_delete", args=[rir_ticket.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_list_view_with_data(self, admin_client, rir_ticket):
        url = reverse("plugins:netbox_rir_manager:rirticket_list")
        response = admin_client.get(url)
        assert response.status_code == 200
        assert rir_ticket.ticket_number.encode() in response.content


@pytest.mark.django_db
class TestRIRConfigSyncView:
    def test_sync_without_key_shows_error(self, admin_client, rir_config):
        url = reverse("plugins:netbox_rir_manager:rirconfig_sync", args=[rir_config.pk])
        response = admin_client.post(url)
        assert response.status_code == 302  # redirect back

    def test_sync_requires_post(self, admin_client, rir_config):
        url = reverse("plugins:netbox_rir_manager:rirconfig_sync", args=[rir_config.pk])
        response = admin_client.get(url)
        assert response.status_code == 405  # Method Not Allowed


@pytest.mark.django_db
class TestUnauthenticatedAccess:
    def test_list_view_requires_login(self):
        from django.test import Client

        client = Client()
        url = reverse("plugins:netbox_rir_manager:rirconfig_list")
        response = client.get(url)
        assert response.status_code == 302  # Redirect to login


@pytest.mark.django_db
class TestRIRNetworkActionViews:
    def test_reassign_get(self, admin_client, rir_network):
        url = reverse("plugins:netbox_rir_manager:rirnetwork_reassign", args=[rir_network.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    def test_reallocate_get(self, admin_client, rir_network):
        url = reverse("plugins:netbox_rir_manager:rirnetwork_reallocate", args=[rir_network.pk])
        response = admin_client.get(url)
        assert response.status_code == 200

    @patch("netbox_rir_manager.views.ARINBackend")
    def test_reassign_simple_success(self, mock_backend_cls, admin_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.create_customer.return_value = {"handle": "C-TEST"}
        mock_backend.reassign_network.return_value = {
            "ticket_number": "TKT-REASSIGN-001",
            "ticket_type": "IPV4_SIMPLE_REASSIGN",
            "ticket_status": "PENDING_REVIEW",
            "raw_data": {},
        }
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins:netbox_rir_manager:rirnetwork_reassign", args=[rir_network.pk])
        response = admin_client.post(
            url,
            {
                "reassignment_type": "simple",
                "customer_name": "Test Customer",
                "city": "Testville",
                "country": "US",
                "start_address": "10.0.0.0",
                "end_address": "10.0.0.255",
            },
        )
        from netbox_rir_manager.models import RIRTicket

        ticket = RIRTicket.objects.get(ticket_number="TKT-REASSIGN-001")
        assert response.status_code == 302
        assert response.url == ticket.get_absolute_url()
        assert ticket.ticket_type == "IPV4_SIMPLE_REASSIGN"

    @patch("netbox_rir_manager.views.ARINBackend")
    def test_reassign_no_api_key(self, mock_backend_cls, admin_client, rir_network):
        """Reassign without an API key should redirect with error."""
        url = reverse("plugins:netbox_rir_manager:rirnetwork_reassign", args=[rir_network.pk])
        response = admin_client.post(
            url,
            {
                "reassignment_type": "detailed",
                "org_handle": "TARGET-ORG",
                "start_address": "10.0.0.0",
                "end_address": "10.0.0.255",
            },
        )
        assert response.status_code == 302
        assert response.url == rir_network.get_absolute_url()

    @patch("netbox_rir_manager.views.ARINBackend")
    def test_remove_success(self, mock_backend_cls, admin_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.remove_network.return_value = True
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins:netbox_rir_manager:rirnetwork_remove", args=[rir_network.pk])
        response = admin_client.post(url)
        assert response.status_code == 302
        assert response.url == rir_network.get_absolute_url()

        from netbox_rir_manager.models import RIRSyncLog

        log = RIRSyncLog.objects.filter(object_handle=rir_network.handle, operation="remove", status="success").first()
        assert log is not None

    @patch("netbox_rir_manager.views.ARINBackend")
    def test_remove_failure(self, mock_backend_cls, admin_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.remove_network.return_value = False
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins:netbox_rir_manager:rirnetwork_remove", args=[rir_network.pk])
        response = admin_client.post(url)
        assert response.status_code == 302
        assert response.url == rir_network.get_absolute_url()

        from netbox_rir_manager.models import RIRSyncLog

        log = RIRSyncLog.objects.filter(object_handle=rir_network.handle, operation="remove", status="error").first()
        assert log is not None

    @patch("netbox_rir_manager.views.ARINBackend")
    def test_delete_arin_success(self, mock_backend_cls, admin_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.delete_network.return_value = {
            "ticket_number": "TKT-DELETE-001",
            "ticket_type": "NET_DELETE_REQUEST",
            "ticket_status": "PENDING_REVIEW",
            "raw_data": {},
        }
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins:netbox_rir_manager:rirnetwork_delete_arin", args=[rir_network.pk])
        response = admin_client.post(url)

        from netbox_rir_manager.models import RIRTicket

        ticket = RIRTicket.objects.get(ticket_number="TKT-DELETE-001")
        assert response.status_code == 302
        assert response.url == ticket.get_absolute_url()

    @patch("netbox_rir_manager.views.ARINBackend")
    def test_delete_arin_failure(self, mock_backend_cls, admin_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.delete_network.return_value = None
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins:netbox_rir_manager:rirnetwork_delete_arin", args=[rir_network.pk])
        response = admin_client.post(url)
        assert response.status_code == 302
        assert response.url == rir_network.get_absolute_url()

    @patch("netbox_rir_manager.views.ARINBackend")
    def test_reallocate_success(self, mock_backend_cls, admin_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.reallocate_network.return_value = {
            "ticket_number": "TKT-REALLOC-001",
            "ticket_type": "IPV4_REALLOCATE",
            "ticket_status": "PENDING_REVIEW",
            "raw_data": {},
        }
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins:netbox_rir_manager:rirnetwork_reallocate", args=[rir_network.pk])
        response = admin_client.post(
            url,
            {
                "org_handle": "TARGET-ORG",
                "start_address": "10.0.0.0",
                "end_address": "10.0.0.255",
            },
        )

        from netbox_rir_manager.models import RIRTicket

        ticket = RIRTicket.objects.get(ticket_number="TKT-REALLOC-001")
        assert response.status_code == 302
        assert response.url == ticket.get_absolute_url()



# Generated by Django 5.2.9 on 2026-02-05

import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("extras", "0134_owner"),
        ("netbox_rir_manager", "0005_rircontact_synced_by_rirnetwork_synced_by_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="RIRTicket",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("ticket_number", models.CharField(max_length=50, unique=True)),
                ("ticket_type", models.CharField(max_length=50)),
                ("status", models.CharField(max_length=30)),
                ("resolution", models.CharField(blank=True, default="", max_length=30)),
                ("created_date", models.DateTimeField()),
                ("resolved_date", models.DateTimeField(blank=True, null=True)),
                ("raw_data", models.JSONField(blank=True, default=dict)),
                (
                    "rir_config",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="tickets",
                        to="netbox_rir_manager.rirconfig",
                    ),
                ),
                (
                    "network",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="tickets",
                        to="netbox_rir_manager.rirnetwork",
                    ),
                ),
                (
                    "submitted_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="tickets",
                        to="netbox_rir_manager.riruserkey",
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
            ],
            options={
                "ordering": ["-created_date"],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
    ]



# Generated by Django 5.2.9 on 2026-02-12 22:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("netbox_rir_manager", "0008_update_rirconfig_objecttype_features"),
    ]

    operations = [
        migrations.AddField(
            model_name="rirnetwork",
            name="net_type",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
    ]



from django.conf import settings
from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel

from netbox_rir_manager.fields import EncryptedCharField


class RIRUserKey(NetBoxModel):
    """Per-user API key for RIR access, scoped to an RIRConfig."""

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="rir_user_keys",
    )
    rir_config = models.ForeignKey(
        "netbox_rir_manager.RIRConfig",
        on_delete=models.CASCADE,
        related_name="user_keys",
        verbose_name="RIR config",
    )
    api_key = EncryptedCharField(max_length=512, verbose_name="API key")

    class Meta:
        ordering = ["user", "rir_config"]
        verbose_name = "RIR user key"
        verbose_name_plural = "RIR user keys"
        constraints = [
            models.UniqueConstraint(fields=["user", "rir_config"], name="unique_user_rir_config"),
        ]

    def __str__(self):
        return f"{self.user.username} - {self.rir_config.name}"

    def get_absolute_url(self):
        return reverse("plugins:netbox_rir_manager:riruserkey", args=[self.pk])



import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcim", "0225_gfk_indexes"),
        ("netbox_rir_manager", "0016_add_location_to_address"),
    ]

    operations = [
        migrations.AlterField(
            model_name="riraddress",
            name="site",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="rir_addresses",
                to="dcim.site",
            ),
        ),
        migrations.AlterField(
            model_name="riraddress",
            name="location",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="rir_address",
                to="dcim.location",
            ),
        ),
        migrations.AddConstraint(
            model_name="riraddress",
            constraint=models.UniqueConstraint(
                condition=models.Q(("location__isnull", True), ("site__isnull", False)),
                fields=("site",),
                name="unique_site_default_address",
            ),
        ),
    ]



# Generated by Django 5.2.9 on 2026-02-11 18:25

from django.db import migrations

import netbox_rir_manager.fields


class Migration(migrations.Migration):
    dependencies = [
        ("netbox_rir_manager", "0006_rirticket"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rirconfig",
            name="api_url",
            field=netbox_rir_manager.fields.LenientURLField(blank=True, default=""),
        ),
    ]



from unittest.mock import patch

import pytest


@pytest.mark.django_db
class TestAutoLinkSignal:
    def test_auto_links_aggregate_on_save(self, rir_config, rir):
        from ipam.models import Aggregate

        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="192.0.2.0/24", rir=rir)
        net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-192-0-2-0-1",
            net_name="EXAMPLE-NET",
            raw_data={"net_blocks": [{"start_address": "192.0.2.0", "cidr_length": 24, "type": "DS"}]},
        )
        net.refresh_from_db()
        assert net.aggregate == agg

    def test_auto_links_prefix_on_save(self, rir_config, rir):
        from ipam.models import Prefix

        from netbox_rir_manager.models import RIRNetwork

        pfx = Prefix.objects.create(prefix="10.0.0.0/8")
        net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-10-0-0-0-1",
            net_name="TEN-NET",
            raw_data={"net_blocks": [{"start_address": "10.0.0.0", "cidr_length": 8, "type": "DS"}]},
        )
        net.refresh_from_db()
        assert net.prefix == pfx

    def test_auto_link_does_not_overwrite_existing(self, rir_config, rir):
        from ipam.models import Aggregate

        from netbox_rir_manager.models import RIRNetwork

        Aggregate.objects.create(prefix="192.0.2.0/24", rir=rir)
        agg2 = Aggregate.objects.create(prefix="10.0.0.0/8", rir=rir)
        net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-192-0-2-0-1",
            net_name="EXAMPLE-NET",
            aggregate=agg2,  # manually set to different aggregate
            raw_data={"net_blocks": [{"start_address": "192.0.2.0", "cidr_length": 24, "type": "DS"}]},
        )
        net.refresh_from_db()
        assert net.aggregate == agg2  # should NOT be overwritten

    def test_auto_link_disabled_by_setting(self, rir_config, rir, settings):
        from ipam.models import Aggregate

        from netbox_rir_manager.models import RIRNetwork

        settings.PLUGINS_CONFIG = {"netbox_rir_manager": {"auto_link_networks": False}}
        Aggregate.objects.create(prefix="192.0.2.0/24", rir=rir)
        net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-DISABLED-1",
            net_name="DISABLED-NET",
            raw_data={"net_blocks": [{"start_address": "192.0.2.0", "cidr_length": 24, "type": "DS"}]},
        )
        net.refresh_from_db()
        assert net.aggregate is None

    def test_no_match_does_nothing(self, rir_config):
        from netbox_rir_manager.models import RIRNetwork

        net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-NOMATCH-1",
            net_name="NO-MATCH-NET",
            raw_data={"net_blocks": [{"start_address": "172.16.0.0", "cidr_length": 12, "type": "DS"}]},
        )
        net.refresh_from_db()
        assert net.aggregate is None
        assert net.prefix is None

    def test_empty_raw_data_does_nothing(self, rir_config):
        from netbox_rir_manager.models import RIRNetwork

        net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-EMPTY-1",
            net_name="EMPTY-NET",
            raw_data={},
        )
        net.refresh_from_db()
        assert net.aggregate is None
        assert net.prefix is None


@pytest.mark.django_db
class TestPrefixDeleteSignal:
    """Tests for remove_rir_network_on_prefix_delete signal."""

    def test_enqueues_removal_for_child_reassignment(self, rir_config, rir_user_key):
        from ipam.models import Prefix

        from netbox_rir_manager.models import RIRNetwork

        pfx = Prefix.objects.create(prefix="10.5.0.0/24")
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-DEL-CHILD-1",
            net_name="DEL-CHILD",
            prefix=pfx,
            # aggregate is None -- this is a child reassignment
        )

        with patch("netbox_rir_manager.jobs.RemoveNetworkJob") as mock_job:
            pfx.delete()

        mock_job.enqueue.assert_called_once()
        call_kwargs = mock_job.enqueue.call_args[1]
        assert call_kwargs["network_id"] is not None
        assert call_kwargs["user_key_id"] == rir_user_key.pk

    def test_skips_parent_allocation(self, rir_config, rir_user_key, rir):
        """RIRNetwork with both aggregate and prefix set should NOT trigger removal."""
        from ipam.models import Aggregate, Prefix

        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="10.6.0.0/20", rir=rir)
        pfx = Prefix.objects.create(prefix="10.6.0.0/20")
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-DEL-PARENT-1",
            net_name="DEL-PARENT",
            prefix=pfx,
            aggregate=agg,  # Has aggregate -- parent allocation
        )

        with patch("netbox_rir_manager.jobs.RemoveNetworkJob") as mock_job:
            pfx.delete()

        mock_job.enqueue.assert_not_called()

    def test_warns_when_no_api_key(self, rir_config):
        """Deletion proceeds but logs a warning when no API key exists."""
        from ipam.models import Prefix

        from netbox_rir_manager.models import RIRNetwork

        pfx = Prefix.objects.create(prefix="10.7.0.0/24")
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-DEL-NOKEY-1",
            net_name="DEL-NOKEY",
            prefix=pfx,
        )

        with patch("netbox_rir_manager.signals.logger") as mock_logger:
            pfx.delete()  # Should not raise

        mock_logger.warning.assert_called_once()
        assert "no API key" in mock_logger.warning.call_args[0][0]

    def test_no_rir_network_does_nothing(self, rir_config):
        """Deleting a prefix with no linked RIRNetwork should not trigger anything."""
        from ipam.models import Prefix

        pfx = Prefix.objects.create(prefix="10.8.0.0/24")

        with patch("netbox_rir_manager.jobs.RemoveNetworkJob") as mock_job:
            pfx.delete()

        mock_job.enqueue.assert_not_called()


@pytest.mark.django_db
class TestAutoReassignStatusGuard:
    """Tests that auto_reassign_prefix only fires for active prefixes."""

    def test_skips_deprecated_prefix(self, rir_config, rir_user_key, rir):
        """A deprecated prefix with site+tenant should NOT trigger reassignment."""
        from ipam.models import Aggregate, Prefix

        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="10.20.0.0/16", rir=rir)
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-10-20-0-0-1",
            net_name="PARENT-NET",
            aggregate=agg,
            auto_reassign=True,
        )

        with patch("netbox_rir_manager.jobs.ReassignJob") as mock_job:
            Prefix.objects.create(prefix="10.20.1.0/24", status="deprecated")

        mock_job.enqueue.assert_not_called()

    def test_skips_reserved_prefix(self, rir_config, rir_user_key, rir):
        """A reserved prefix should NOT trigger reassignment."""
        from ipam.models import Aggregate, Prefix

        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="10.21.0.0/16", rir=rir)
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-10-21-0-0-1",
            net_name="PARENT-NET-2",
            aggregate=agg,
            auto_reassign=True,
        )

        with patch("netbox_rir_manager.jobs.ReassignJob") as mock_job:
            Prefix.objects.create(prefix="10.21.1.0/24", status="reserved")

        mock_job.enqueue.assert_not_called()


@pytest.mark.django_db
class TestPrefixDeactivateSignal:
    """Tests for remove_network_on_prefix_deactivate signal."""

    def test_enqueues_removal_on_deactivation(self, rir_config, rir_user_key):
        """Changing prefix to deprecated should enqueue ARIN network removal."""
        from ipam.models import Prefix

        from netbox_rir_manager.models import RIRNetwork

        pfx = Prefix.objects.create(prefix="10.30.0.0/24", status="active")
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-DEACT-1",
            net_name="DEACT-NET",
            prefix=pfx,
        )

        with patch("netbox_rir_manager.jobs.RemoveNetworkJob") as mock_job:
            pfx.status = "deprecated"
            pfx.save()

        mock_job.enqueue.assert_called_once()
        call_kwargs = mock_job.enqueue.call_args[1]
        assert call_kwargs["network_id"] is not None
        assert call_kwargs["user_key_id"] == rir_user_key.pk

    def test_skips_active_prefix(self, rir_config, rir_user_key):
        """Saving a prefix that stays active should NOT trigger removal."""
        from ipam.models import Prefix

        from netbox_rir_manager.models import RIRNetwork

        pfx = Prefix.objects.create(prefix="10.31.0.0/24", status="active")
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-ACTIVE-1",
            net_name="ACTIVE-NET",
            prefix=pfx,
        )

        with patch("netbox_rir_manager.jobs.RemoveNetworkJob") as mock_job:
            pfx.description = "updated"
            pfx.save()

        mock_job.enqueue.assert_not_called()

    def test_skips_newly_created_prefix(self, rir_config, rir_user_key):
        """A newly created deprecated prefix should NOT trigger removal."""
        from ipam.models import Prefix

        with patch("netbox_rir_manager.jobs.RemoveNetworkJob") as mock_job:
            Prefix.objects.create(prefix="10.32.0.0/24", status="deprecated")

        mock_job.enqueue.assert_not_called()

    def test_skips_parent_allocation(self, rir_config, rir_user_key, rir):
        """RIRNetwork with aggregate set should NOT trigger removal on deactivation."""
        from ipam.models import Aggregate, Prefix

        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="10.33.0.0/20", rir=rir)
        pfx = Prefix.objects.create(prefix="10.33.0.0/20", status="active")
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-DEACT-PARENT-1",
            net_name="DEACT-PARENT",
            prefix=pfx,
            aggregate=agg,
        )

        with patch("netbox_rir_manager.jobs.RemoveNetworkJob") as mock_job:
            pfx.status = "deprecated"
            pfx.save()

        mock_job.enqueue.assert_not_called()

    def test_warns_when_no_api_key(self, rir_config):
        """Deactivation logs a warning when no API key exists."""
        from ipam.models import Prefix

        from netbox_rir_manager.models import RIRNetwork

        pfx = Prefix.objects.create(prefix="10.34.0.0/24", status="active")
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-DEACT-NOKEY-1",
            net_name="DEACT-NOKEY",
            prefix=pfx,
        )

        with patch("netbox_rir_manager.signals.logger") as mock_logger:
            pfx.status = "deprecated"
            pfx.save()

        mock_logger.warning.assert_called_once()
        assert "no API key" in mock_logger.warning.call_args[0][0]



# Generated by Django 5.2.9 on 2026-02-17 04:24

from django.db import migrations, models


def dedup_addresses(apps, schema_editor):
    """Merge duplicate RIRAddress rows before adding unique constraint."""
    RIRAddress = apps.get_model("netbox_rir_manager", "RIRAddress")
    dupes = (
        RIRAddress.objects.values("street_address", "city", "state_province", "postal_code", "country")
        .annotate(cnt=models.Count("id"))
        .filter(cnt__gt=1)
    )
    for group in dupes:
        addrs = list(
            RIRAddress.objects.filter(
                street_address=group["street_address"],
                city=group["city"],
                state_province=group["state_province"],
                postal_code=group["postal_code"],
                country=group["country"],
            ).order_by("pk")
        )
        keeper = addrs[0]
        for dup in addrs[1:]:
            # Re-point all FKs to the keeper
            for related in ("rir_organizations", "rir_contacts", "rir_customers"):
                getattr(dup, related).update(address=keeper)
            dup.delete()


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ("dcim", "0225_gfk_indexes"),
        ("extras", "0134_owner"),
        ("netbox_rir_manager", "0014_riraddress_refactor"),
    ]

    operations = [
        migrations.RunPython(dedup_addresses, migrations.RunPython.noop),
        migrations.AddConstraint(
            model_name="riraddress",
            constraint=models.UniqueConstraint(
                fields=("street_address", "city", "state_province", "postal_code", "country"),
                name="unique_rir_address",
            ),
        ),
    ]



from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel

from netbox_rir_manager.choices import SyncOperationChoices, SyncStatusChoices


class RIRSyncLog(NetBoxModel):
    """Audit log for sync operations."""

    rir_config = models.ForeignKey(
        "netbox_rir_manager.RIRConfig",
        on_delete=models.CASCADE,
        related_name="sync_logs",
        verbose_name="RIR config",
    )
    operation = models.CharField(max_length=50, choices=SyncOperationChoices)
    object_type = models.CharField(max_length=50)
    object_handle = models.CharField(max_length=100)
    status = models.CharField(max_length=20, choices=SyncStatusChoices)
    message = models.TextField(blank=True, default="")

    class Meta:
        ordering = ["-created"]
        verbose_name = "RIR sync log"
        verbose_name_plural = "RIR sync logs"

    def __str__(self):
        return f"{self.operation} {self.object_handle} ({self.status})"

    def get_absolute_url(self):
        return reverse("plugins:netbox_rir_manager:rirsynclog", args=[self.pk])



import ipaddress
import json

from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils import timezone
from django.views import View
from netbox.object_actions import (
    AddObject,
    BulkDelete,
    BulkEdit,
    BulkExport,
    BulkImport,
    DeleteObject,
    ObjectAction,
)
from netbox.views import generic

from netbox_rir_manager.backends.arin import ARINBackend
from netbox_rir_manager.choices import normalize_ticket_status
from netbox_rir_manager.filtersets import (
    RIRAddressFilterSet,
    RIRConfigFilterSet,
    RIRContactFilterSet,
    RIRCustomerFilterSet,
    RIRNetworkFilterSet,
    RIROrganizationFilterSet,
    RIRSyncLogFilterSet,
    RIRTicketFilterSet,
    RIRUserKeyFilterSet,
)
from netbox_rir_manager.forms import (
    RIRAddressFilterForm,
    RIRAddressForm,
    RIRConfigBulkEditForm,
    RIRConfigFilterForm,
    RIRConfigForm,
    RIRConfigImportForm,
    RIRContactFilterForm,
    RIRContactForm,
    RIRCustomerFilterForm,
    RIRNetworkFilterForm,
    RIRNetworkForm,
    RIRNetworkReallocateForm,
    RIRNetworkReassignForm,
    RIROrganizationFilterForm,
    RIROrganizationForm,
    RIRTicketFilterForm,
    RIRUserKeyFilterForm,
    RIRUserKeyForm,
)
from netbox_rir_manager.models import (
    RIRAddress,
    RIRConfig,
    RIRContact,
    RIRCustomer,
    RIRNetwork,
    RIROrganization,
    RIRSyncLog,
    RIRTicket,
    RIRUserKey,
)
from netbox_rir_manager.tables import (
    RIRAddressTable,
    RIRConfigTable,
    RIRContactTable,
    RIRCustomerTable,
    RIRNetworkTable,
    RIROrganizationTable,
    RIRSyncLogTable,
    RIRTicketTable,
    RIRUserKeyTable,
)


# --- RIRConfig Views ---
class BulkSync(ObjectAction):
    name = "bulk_sync"
    label = "Sync Selected"
    multi = True
    permissions_required = {"change"}
    template_name = "netbox_rir_manager/buttons/bulk_sync.html"


class RIRConfigListView(generic.ObjectListView):
    queryset = RIRConfig.objects.all()
    table = RIRConfigTable
    filterset = RIRConfigFilterSet
    filterset_form = RIRConfigFilterForm
    actions = (AddObject, BulkImport, BulkExport, BulkEdit, BulkSync, BulkDelete)


class RIRConfigView(generic.ObjectView):
    queryset = RIRConfig.objects.all()


class RIRConfigEditView(generic.ObjectEditView):
    queryset = RIRConfig.objects.all()
    form = RIRConfigForm


class RIRConfigDeleteView(generic.ObjectDeleteView):
    queryset = RIRConfig.objects.all()


class RIRConfigBulkImportView(generic.BulkImportView):
    queryset = RIRConfig.objects.all()
    model_form = RIRConfigImportForm


class RIRConfigBulkEditView(generic.BulkEditView):
    queryset = RIRConfig.objects.all()
    filterset = RIRConfigFilterSet
    table = RIRConfigTable
    form = RIRConfigBulkEditForm


class RIRConfigBulkDeleteView(generic.BulkDeleteView):
    queryset = RIRConfig.objects.all()
    filterset = RIRConfigFilterSet
    table = RIRConfigTable


class RIRConfigBulkSyncView(LoginRequiredMixin, View):
    """Trigger sync jobs for multiple RIR configs at once."""

    def post(self, request):
        from netbox_rir_manager.jobs import SyncRIRConfigJob

        pk_list = [int(pk) for pk in request.POST.getlist("pk")]

        if "_confirm" in request.POST:
            configs = RIRConfig.objects.filter(pk__in=pk_list)
            synced = []
            skipped = []
            for config in configs:
                user_key = RIRUserKey.objects.filter(user=request.user, rir_config=config).first()
                if not user_key:
                    skipped.append(config.name)
                    continue
                SyncRIRConfigJob.enqueue(instance=config, user=request.user, user_id=request.user.pk)
                synced.append(config.name)

            if synced:
                messages.success(request, f"Sync jobs queued for: {', '.join(synced)}")
            if skipped:
                messages.warning(request, f"Skipped (no API key): {', '.join(skipped)}")

            return redirect(reverse("plugins:netbox_rir_manager:rirconfig_list"))

        configs = RIRConfig.objects.filter(pk__in=pk_list)
        table = RIRConfigTable(configs, orderable=False)
        if not table.rows:
            messages.warning(request, "No configs were selected.")
            return redirect(reverse("plugins:netbox_rir_manager:rirconfig_list"))

        return render(
            request,
            "netbox_rir_manager/rirconfig_bulk_sync.html",
            {
                "table": table,
                "pk_list": pk_list,
                "return_url": reverse("plugins:netbox_rir_manager:rirconfig_list"),
            },
        )


# --- RIROrganization Views ---
class RIROrganizationListView(generic.ObjectListView):
    queryset = RIROrganization.objects.select_related("address").all()
    table = RIROrganizationTable
    filterset = RIROrganizationFilterSet
    filterset_form = RIROrganizationFilterForm


class RIROrganizationView(generic.ObjectView):
    queryset = RIROrganization.objects.select_related("address").all()


class RIROrganizationEditView(generic.ObjectEditView):
    queryset = RIROrganization.objects.all()
    form = RIROrganizationForm


class RIROrganizationDeleteView(generic.ObjectDeleteView):
    queryset = RIROrganization.objects.all()


# --- RIRContact Views ---
class RIRContactListView(generic.ObjectListView):
    queryset = RIRContact.objects.select_related("address").all()
    table = RIRContactTable
    filterset = RIRContactFilterSet
    filterset_form = RIRContactFilterForm


class RIRContactView(generic.ObjectView):
    queryset = RIRContact.objects.select_related("address").all()


class RIRContactEditView(generic.ObjectEditView):
    queryset = RIRContact.objects.all()
    form = RIRContactForm


class RIRContactDeleteView(generic.ObjectDeleteView):
    queryset = RIRContact.objects.all()


# --- RIRCustomer Views ---
class RIRCustomerListView(generic.ObjectListView):
    queryset = RIRCustomer.objects.select_related("address").all()
    table = RIRCustomerTable
    filterset = RIRCustomerFilterSet
    filterset_form = RIRCustomerFilterForm
    actions = (BulkExport, BulkDelete)


class RIRCustomerView(generic.ObjectView):
    queryset = RIRCustomer.objects.select_related("address").all()
    actions = (DeleteObject,)


class RIRCustomerDeleteView(generic.ObjectDeleteView):
    queryset = RIRCustomer.objects.all()


# --- RIRNetwork Views ---
class RIRNetworkListView(generic.ObjectListView):
    queryset = RIRNetwork.objects.all()
    table = RIRNetworkTable
    filterset = RIRNetworkFilterSet
    filterset_form = RIRNetworkFilterForm


class RIRNetworkView(generic.ObjectView):
    queryset = RIRNetwork.objects.all()


class RIRNetworkEditView(generic.ObjectEditView):
    queryset = RIRNetwork.objects.all()
    form = RIRNetworkForm


class RIRNetworkDeleteView(generic.ObjectDeleteView):
    queryset = RIRNetwork.objects.all()


# --- RIRAddress Views ---
class RIRAddressListView(generic.ObjectListView):
    queryset = RIRAddress.objects.all()
    table = RIRAddressTable
    filterset = RIRAddressFilterSet
    filterset_form = RIRAddressFilterForm


class RIRAddressView(generic.ObjectView):
    queryset = RIRAddress.objects.all()


class RIRAddressEditView(generic.ObjectEditView):
    queryset = RIRAddress.objects.all()
    form = RIRAddressForm


class RIRAddressDeleteView(generic.ObjectDeleteView):
    queryset = RIRAddress.objects.all()


# --- RIRSyncLog Views ---
class RIRSyncLogListView(generic.ObjectListView):
    queryset = RIRSyncLog.objects.all()
    table = RIRSyncLogTable
    filterset = RIRSyncLogFilterSet


class RIRSyncLogView(generic.ObjectView):
    queryset = RIRSyncLog.objects.all()


class RIRSyncLogDeleteView(generic.ObjectDeleteView):
    queryset = RIRSyncLog.objects.all()


# --- RIRUserKey Views ---
class RIRUserKeyListView(generic.ObjectListView):
    queryset = RIRUserKey.objects.all()
    table = RIRUserKeyTable
    filterset = RIRUserKeyFilterSet
    filterset_form = RIRUserKeyFilterForm


class RIRUserKeyView(generic.ObjectView):
    queryset = RIRUserKey.objects.all()


class RIRUserKeyEditView(generic.ObjectEditView):
    queryset = RIRUserKey.objects.all()
    form = RIRUserKeyForm


class RIRUserKeyDeleteView(generic.ObjectDeleteView):
    queryset = RIRUserKey.objects.all()


# --- RIRTicket Views ---
class RIRTicketListView(generic.ObjectListView):
    queryset = RIRTicket.objects.all()
    table = RIRTicketTable
    filterset = RIRTicketFilterSet
    filterset_form = RIRTicketFilterForm


class RIRTicketView(generic.ObjectView):
    queryset = RIRTicket.objects.all()


class RIRTicketDeleteView(generic.ObjectDeleteView):
    queryset = RIRTicket.objects.all()


# --- Sync Trigger View ---
class RIRConfigSyncView(LoginRequiredMixin, View):
    """Trigger a background sync job for an RIRConfig."""

    def post(self, request, pk):
        from netbox_rir_manager.jobs import SyncRIRConfigJob
        from netbox_rir_manager.models import RIRUserKey as RIRUserKeyModel

        rir_config = get_object_or_404(RIRConfig, pk=pk)

        if not RIRUserKeyModel.objects.filter(user=request.user, rir_config=rir_config).exists():
            messages.error(request, "You don't have an API key configured for this RIR config.")
            return redirect(rir_config.get_absolute_url())

        SyncRIRConfigJob.enqueue(
            instance=rir_config,
            user=request.user,
            user_id=request.user.pk,
        )
        messages.success(request, f"Sync job queued for {rir_config.name}.")
        return_url = request.GET.get("return_url") or request.POST.get("return_url")
        return redirect(return_url or rir_config.get_absolute_url())


class RIRTicketRefreshView(LoginRequiredMixin, View):
    """Refresh ticket status from ARIN (placeholder)."""

    def post(self, request, pk):
        ticket = get_object_or_404(RIRTicket, pk=pk)
        messages.info(request, f"Ticket {ticket.ticket_number} status is: {ticket.get_status_display()}")
        return redirect(ticket.get_absolute_url())


class RIRNetworkReassignView(LoginRequiredMixin, View):
    """Reassign a subnet from this network."""

    def get(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        form = RIRNetworkReassignForm()
        return render(
            request,
            "netbox_rir_manager/rirnetwork_reassign.html",
            {
                "object": network,
                "form": form,
            },
        )

    def post(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        form = RIRNetworkReassignForm(request.POST)
        if not form.is_valid():
            return render(
                request,
                "netbox_rir_manager/rirnetwork_reassign.html",
                {
                    "object": network,
                    "form": form,
                },
            )

        user_key = RIRUserKey.objects.filter(
            user=request.user,
            rir_config=network.rir_config,
        ).first()
        if not user_key:
            messages.error(request, "No API key configured for this RIR config.")
            return redirect(network.get_absolute_url())

        backend = ARINBackend.from_rir_config(network.rir_config, api_key=user_key.api_key)

        rtype = form.cleaned_data["reassignment_type"]
        if rtype == "simple":
            customer_data = {
                "customer_name": form.cleaned_data["customer_name"],
                "street_address": form.cleaned_data.get("street_address", ""),
                "city": form.cleaned_data["city"],
                "state_province": form.cleaned_data.get("state_province", ""),
                "postal_code": form.cleaned_data.get("postal_code", ""),
                "country": form.cleaned_data["country"],
            }
            customer_result = backend.create_customer(network.handle, customer_data)
            if customer_result is None:
                messages.error(request, "Failed to create customer at ARIN.")
                RIRSyncLog.objects.create(
                    rir_config=network.rir_config,
                    operation="create",
                    object_type="customer",
                    object_handle=network.handle,
                    status="error",
                    message="Failed to create customer",
                )
                return redirect(network.get_absolute_url())

            addr = RIRAddress.objects.create(
                street_address=customer_data.get("street_address", ""),
                city=customer_data.get("city", ""),
                state_province=customer_data.get("state_province", ""),
                postal_code=customer_data.get("postal_code", ""),
                country=customer_data.get("country", ""),
            )
            RIRCustomer.objects.create(
                rir_config=network.rir_config,
                handle=customer_result["handle"],
                customer_name=customer_data["customer_name"],
                address=addr,
                network=network,
                raw_data=customer_result,
                created_date=timezone.now(),
            )

        net_data = {
            "net_name": form.cleaned_data.get("net_name", ""),
            "start_address": str(form.cleaned_data["start_address"]),
            "end_address": str(form.cleaned_data["end_address"]),
        }
        if rtype == "simple":
            net_data["customer_handle"] = customer_result["handle"]
        else:
            net_data["org_handle"] = form.cleaned_data["org_handle"]

        result = backend.reassign_network(network.handle, net_data)
        if result is None:
            messages.error(request, "Reassignment failed at ARIN.")
            RIRSyncLog.objects.create(
                rir_config=network.rir_config,
                operation="reassign",
                object_type="network",
                object_handle=network.handle,
                status="error",
                message="Reassignment failed",
            )
            return redirect(network.get_absolute_url())

        ticket = RIRTicket.objects.create(
            rir_config=network.rir_config,
            ticket_number=result.get("ticket_number", ""),
            ticket_type=result.get("ticket_type", "IPV4_SIMPLE_REASSIGN"),
            status=normalize_ticket_status(result.get("ticket_status", "")),
            network=network,
            submitted_by=user_key,
            created_date=timezone.now(),
            raw_data=result.get("raw_data", {}),
        )
        RIRSyncLog.objects.create(
            rir_config=network.rir_config,
            operation="reassign",
            object_type="network",
            object_handle=network.handle,
            status="success",
            message=f"Reassignment submitted, ticket {ticket.ticket_number}",
        )
        messages.success(request, f"Reassignment submitted. Ticket: {ticket.ticket_number}")
        return redirect(ticket.get_absolute_url())


class RIRNetworkReallocateView(LoginRequiredMixin, View):
    """Reallocate a subnet from this network."""

    def get(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        form = RIRNetworkReallocateForm()
        return render(
            request,
            "netbox_rir_manager/rirnetwork_reallocate.html",
            {
                "object": network,
                "form": form,
            },
        )

    def post(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        form = RIRNetworkReallocateForm(request.POST)
        if not form.is_valid():
            return render(
                request,
                "netbox_rir_manager/rirnetwork_reallocate.html",
                {
                    "object": network,
                    "form": form,
                },
            )

        user_key = RIRUserKey.objects.filter(
            user=request.user,
            rir_config=network.rir_config,
        ).first()
        if not user_key:
            messages.error(request, "No API key configured for this RIR config.")
            return redirect(network.get_absolute_url())

        backend = ARINBackend.from_rir_config(network.rir_config, api_key=user_key.api_key)
        net_data = {
            "org_handle": form.cleaned_data["org_handle"],
            "net_name": form.cleaned_data.get("net_name", ""),
            "start_address": str(form.cleaned_data["start_address"]),
            "end_address": str(form.cleaned_data["end_address"]),
        }
        result = backend.reallocate_network(network.handle, net_data)
        if result is None:
            messages.error(request, "Reallocation failed at ARIN.")
            RIRSyncLog.objects.create(
                rir_config=network.rir_config,
                operation="reallocate",
                object_type="network",
                object_handle=network.handle,
                status="error",
                message="Reallocation failed",
            )
            return redirect(network.get_absolute_url())

        ticket = RIRTicket.objects.create(
            rir_config=network.rir_config,
            ticket_number=result.get("ticket_number", ""),
            ticket_type=result.get("ticket_type", "IPV4_REALLOCATE"),
            status=normalize_ticket_status(result.get("ticket_status", "")),
            network=network,
            submitted_by=user_key,
            created_date=timezone.now(),
            raw_data=result.get("raw_data", {}),
        )
        RIRSyncLog.objects.create(
            rir_config=network.rir_config,
            operation="reallocate",
            object_type="network",
            object_handle=network.handle,
            status="success",
            message=f"Reallocation submitted, ticket {ticket.ticket_number}",
        )
        messages.success(request, f"Reallocation submitted. Ticket: {ticket.ticket_number}")
        return redirect(ticket.get_absolute_url())


class RIRNetworkRemoveView(LoginRequiredMixin, View):
    """Remove a reassigned/reallocated network from ARIN (with confirmation page)."""

    def get(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        return render(
            request,
            "netbox_rir_manager/rirnetwork_confirm_remove.html",
            {"object": network},
        )

    def post(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        user_key = RIRUserKey.objects.filter(
            user=request.user,
            rir_config=network.rir_config,
        ).first()
        if not user_key:
            messages.error(request, "No API key configured for this RIR config.")
            return redirect(network.get_absolute_url())

        backend = ARINBackend.from_rir_config(network.rir_config, api_key=user_key.api_key)
        success = backend.remove_network(network.handle)
        if success:
            RIRSyncLog.objects.create(
                rir_config=network.rir_config,
                operation="remove",
                object_type="network",
                object_handle=network.handle,
                status="success",
                message=f"Removed network {network.handle} from ARIN",
            )
            messages.success(request, f"Network {network.handle} removed from ARIN.")
        else:
            RIRSyncLog.objects.create(
                rir_config=network.rir_config,
                operation="remove",
                object_type="network",
                object_handle=network.handle,
                status="error",
                message="Failed to remove network from ARIN",
            )
            messages.error(request, "Failed to remove network from ARIN.")
        return redirect(network.get_absolute_url())


class RIRNetworkDeleteARINView(LoginRequiredMixin, View):
    """Delete a network at ARIN (creates a ticket, with confirmation page)."""

    def get(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        return render(
            request,
            "netbox_rir_manager/rirnetwork_confirm_delete_arin.html",
            {"object": network},
        )

    def post(self, request, pk):
        network = get_object_or_404(RIRNetwork, pk=pk)
        user_key = RIRUserKey.objects.filter(
            user=request.user,
            rir_config=network.rir_config,
        ).first()
        if not user_key:
            messages.error(request, "No API key configured for this RIR config.")
            return redirect(network.get_absolute_url())

        backend = ARINBackend.from_rir_config(network.rir_config, api_key=user_key.api_key)
        result = backend.delete_network(network.handle)
        if result is None:
            RIRSyncLog.objects.create(
                rir_config=network.rir_config,
                operation="delete",
                object_type="network",
                object_handle=network.handle,
                status="error",
                message="Failed to delete network at ARIN",
            )
            messages.error(request, "Failed to delete network at ARIN.")
            return redirect(network.get_absolute_url())

        ticket = RIRTicket.objects.create(
            rir_config=network.rir_config,
            ticket_number=result.get("ticket_number", ""),
            ticket_type="NET_DELETE_REQUEST",
            status=normalize_ticket_status(result.get("ticket_status", "")),
            network=network,
            submitted_by=user_key,
            created_date=timezone.now(),
            raw_data=result.get("raw_data", {}),
        )
        RIRSyncLog.objects.create(
            rir_config=network.rir_config,
            operation="delete",
            object_type="network",
            object_handle=network.handle,
            status="success",
            message=f"Delete request submitted, ticket {ticket.ticket_number}",
        )
        messages.success(request, f"Delete request submitted. Ticket: {ticket.ticket_number}")
        return redirect(ticket.get_absolute_url())


# --- Aggregate Sync View (from Aggregate detail page) ---
class AggregateSyncView(LoginRequiredMixin, View):
    """Sync a single aggregate from ARIN."""

    def post(self, request, pk):
        from ipam.models import Aggregate

        aggregate = get_object_or_404(Aggregate, pk=pk)

        # Find the RIR config for this aggregate's RIR
        rir_config = RIRConfig.objects.filter(rir=aggregate.rir, is_active=True).first()
        if not rir_config:
            messages.error(request, "No active RIR config found for this aggregate's RIR.")
            return redirect(aggregate.get_absolute_url())

        # Get user's API key
        user_key = RIRUserKey.objects.filter(user=request.user, rir_config=rir_config).first()
        if not user_key:
            messages.error(request, "No API key configured for this RIR config.")
            return redirect(aggregate.get_absolute_url())

        backend = ARINBackend.from_rir_config(rir_config, api_key=user_key.api_key)

        # Sync this specific aggregate
        network = aggregate.prefix
        start_address = str(network.network)
        end_address = str(network.broadcast)

        net_data = backend.find_net(start_address, end_address)
        if net_data is None:
            messages.warning(request, "No matching network found at ARIN for this aggregate.")
            RIRSyncLog.objects.create(
                rir_config=rir_config,
                operation="sync",
                object_type="network",
                object_handle=str(aggregate.prefix),
                status="skipped",
                message=f"No ARIN network found for {aggregate.prefix}",
            )
            return redirect(aggregate.get_absolute_url())

        net, created = RIRNetwork.sync_from_arin(
            net_data,
            rir_config,
            aggregate=aggregate,
            user_key=user_key,
        )

        RIRSyncLog.objects.create(
            rir_config=rir_config,
            operation="sync",
            object_type="network",
            object_handle=net_data["handle"],
            status="success",
            message=f"{'Created' if created else 'Updated'} network {net_data['handle']}",
        )

        action = "Created" if created else "Updated"
        messages.success(request, f"{action} RIR network {net.handle} from ARIN.")
        return redirect(aggregate.get_absolute_url())


# --- Prefix Sync View (from Prefix detail page) ---
class PrefixSyncView(LoginRequiredMixin, View):
    """Sync a single prefix from ARIN using mostSpecificNet."""

    def post(self, request, pk):
        from ipam.models import Aggregate, Prefix

        prefix = get_object_or_404(Prefix, pk=pk)

        # Find parent aggregate to determine RIR config
        agg = Aggregate.objects.filter(prefix__net_contains_or_equals=prefix.prefix).first()
        if not agg:
            messages.error(request, "No parent aggregate found for this prefix.")
            return redirect(prefix.get_absolute_url())

        rir_config = RIRConfig.objects.filter(rir=agg.rir, is_active=True).first()
        if not rir_config:
            messages.error(request, "No active RIR config found for this prefix's RIR.")
            return redirect(prefix.get_absolute_url())

        user_key = RIRUserKey.objects.filter(user=request.user, rir_config=rir_config).first()
        if not user_key:
            messages.error(request, "No API key configured for this RIR config.")
            return redirect(prefix.get_absolute_url())

        backend = ARINBackend.from_rir_config(rir_config, api_key=user_key.api_key)

        network = ipaddress.ip_network(str(prefix.prefix), strict=False)
        start_address = str(network.network_address)
        end_address = str(network.broadcast_address)

        net_data = backend.find_net(start_address, end_address)
        if net_data is None:
            messages.warning(request, "No matching network found at ARIN for this prefix.")
            RIRSyncLog.objects.create(
                rir_config=rir_config,
                operation="sync",
                object_type="network",
                object_handle=str(prefix.prefix),
                status="skipped",
                message=f"No ARIN network found for {prefix.prefix}",
            )
            return redirect(prefix.get_absolute_url())

        # Check if this is actually a child net (not the parent aggregate's net)
        parent_network = RIRNetwork.objects.filter(aggregate=agg).first()
        if parent_network and net_data["handle"] == parent_network.handle:
            messages.info(
                request,
                "No separate reassignment found at ARIN -- this prefix is covered by the parent allocation.",
            )
            return redirect(prefix.get_absolute_url())

        net, created = RIRNetwork.sync_from_arin(
            net_data,
            rir_config,
            prefix=prefix,
            user_key=user_key,
        )

        RIRSyncLog.objects.create(
            rir_config=rir_config,
            operation="sync",
            object_type="network",
            object_handle=net_data["handle"],
            status="success",
            message=f"{'Created' if created else 'Updated'} network {net_data['handle']} for prefix {prefix.prefix}",
        )

        action = "Created" if created else "Updated"
        messages.success(request, f"{action} RIR network {net.handle} from ARIN.")
        return redirect(prefix.get_absolute_url())


# --- Prefix Reassign View (from Prefix detail page) ---
class PrefixReassignView(LoginRequiredMixin, View):
    """Reassign a prefix at ARIN from the Prefix detail page."""

    def get(self, request, pk):
        from ipam.models import Aggregate, Prefix

        prefix = get_object_or_404(Prefix, pk=pk)

        # Find parent RIRNetwork
        agg = Aggregate.objects.filter(prefix__net_contains_or_equals=prefix.prefix).first()
        parent_network = RIRNetwork.objects.filter(aggregate=agg).first() if agg else None

        # Pre-fill form
        network = ipaddress.ip_network(str(prefix.prefix), strict=False)
        initial = {
            "start_address": str(network.network_address),
            "end_address": str(network.broadcast_address),
        }

        # Pre-fill from tenant
        if prefix.tenant:
            initial["customer_name"] = prefix.tenant.name

            # Check if tenant has a linked RIROrganization
            rir_org = RIROrganization.objects.filter(tenant=prefix.tenant).first()
            if rir_org:
                initial["reassignment_type"] = "detailed"
                initial["org_handle"] = rir_org.handle

        # Pre-fill from site address
        site = getattr(prefix, "_site", None) or getattr(prefix, "site", None)
        if site is None:
            from dcim.models import Site

            scope = getattr(prefix, "scope", None)
            if isinstance(scope, Site):
                site = scope

        if site:
            site_address = RIRAddress.get_for_site(site)
            if site_address:
                initial["street_address"] = site_address.street_address
                initial["city"] = site_address.city
                initial["state_province"] = site_address.state_province
                initial["postal_code"] = site_address.postal_code
                initial["country"] = site_address.country

        # Generate net_name from tenant + prefix
        if prefix.tenant:
            initial["net_name"] = f"{prefix.tenant.name}-{prefix.prefix}"

        form = RIRNetworkReassignForm(initial=initial)

        return render(
            request,
            "netbox_rir_manager/prefix_reassign.html",
            {
                "object": prefix,
                "form": form,
                "parent_network": parent_network,
            },
        )

    def post(self, request, pk):
        from ipam.models import Aggregate, Prefix

        prefix = get_object_or_404(Prefix, pk=pk)
        form = RIRNetworkReassignForm(request.POST)

        # Find parent RIRNetwork
        agg = Aggregate.objects.filter(prefix__net_contains_or_equals=prefix.prefix).first()
        parent_network = RIRNetwork.objects.filter(aggregate=agg).first() if agg else None

        if not form.is_valid():
            return render(
                request,
                "netbox_rir_manager/prefix_reassign.html",
                {
                    "object": prefix,
                    "form": form,
                    "parent_network": parent_network,
                },
            )

        if not parent_network:
            messages.error(request, "No parent RIR network found for this prefix.")
            return redirect(prefix.get_absolute_url())

        user_key = RIRUserKey.objects.filter(
            user=request.user,
            rir_config=parent_network.rir_config,
        ).first()
        if not user_key:
            messages.error(request, "No API key configured for this RIR config.")
            return redirect(prefix.get_absolute_url())

        # Enqueue as background job
        from netbox_rir_manager.jobs import ReassignJob

        ReassignJob.enqueue(
            instance=parent_network.rir_config,
            user=request.user,
            prefix_id=prefix.pk,
            user_key_id=user_key.pk,
        )

        messages.success(request, f"Reassignment job queued for prefix {prefix.prefix}.")
        return redirect(prefix.get_absolute_url())


# --- Site Address Resolve Views ---
class SiteAddressResolveModalView(LoginRequiredMixin, View):
    """Return HTMX modal content with geocoding candidates for a Site."""

    template_name = "netbox_rir_manager/htmx/site_address_resolve_modal.html"

    def get(self, request, pk):
        from dcim.models import Site

        from netbox_rir_manager.services.geocoding import resolve_site_address_candidates

        site = get_object_or_404(Site, pk=pk)
        candidates = resolve_site_address_candidates(site)

        # Pre-fill search field based on which source produced results:
        # reverse geocode from coords → show coords, forward geocode → show address
        if site.latitude and site.longitude:
            search_query = f"{site.latitude}, {site.longitude}"
        elif site.physical_address:
            search_query = site.physical_address
        else:
            search_query = ""

        return render(
            request,
            self.template_name,
            {
                "site": site,
                "candidates": candidates,
                "search_query": search_query,
            },
        )

    def post(self, request, pk):
        from dcim.models import Site

        from netbox_rir_manager.services.geocoding import resolve_site_address_candidates

        site = get_object_or_404(Site, pk=pk)
        query = request.POST.get("query", "").strip()
        candidates = resolve_site_address_candidates(site, query=query or None)

        return render(
            request,
            self.template_name,
            {
                "site": site,
                "candidates": candidates,
                "search_query": query,
            },
        )


class SiteAddressSelectView(LoginRequiredMixin, View):
    """Create an RIRAddress from a selected geocoding candidate."""

    def post(self, request, pk):
        from dcim.models import Site

        site = get_object_or_404(Site, pk=pk)

        street_address = request.POST.get("street_address", "")
        city = request.POST.get("city", "")
        state_province = request.POST.get("state_province", "")
        postal_code = request.POST.get("postal_code", "")
        country = request.POST.get("country", "")
        raw_geocode_str = request.POST.get("raw_geocode", "{}")

        try:
            raw_geocode = json.loads(raw_geocode_str)
        except (json.JSONDecodeError, TypeError):
            raw_geocode = {}

        # Delete existing site-level address (not location-specific ones)
        RIRAddress.objects.filter(site=site, location__isnull=True).delete()

        # Use get_or_create on address fields to handle unique constraint
        address, created = RIRAddress.objects.get_or_create(
            street_address=street_address,
            city=city,
            state_province=state_province,
            postal_code=postal_code,
            country=country,
            defaults={
                "raw_geocode": raw_geocode,
                "auto_resolved": True,
                "last_resolved": timezone.now(),
            },
        )

        # Assign site FK (whether newly created or existing)
        address.site = site
        address.auto_resolved = True
        address.last_resolved = timezone.now()
        if raw_geocode:
            address.raw_geocode = raw_geocode
        address.save()

        return HttpResponse(status=204, headers={"HX-Redirect": site.get_absolute_url()})



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from netbox_rir_manager.models import RIRConfig


class RIRBackend(ABC):
    """Abstract base class for RIR API backends."""

    name: str

    @abstractmethod
    def authenticate(self, rir_config: RIRConfig) -> bool:
        """Validate credentials and establish connection."""
        ...

    @abstractmethod
    def get_organization(self, handle: str) -> dict[str, Any] | None:
        """Retrieve organization details by handle."""
        ...

    @abstractmethod
    def get_network(self, handle: str) -> dict[str, Any] | None:
        """Retrieve network/prefix details."""
        ...

    @abstractmethod
    def get_poc(self, handle: str) -> dict[str, Any] | None:
        """Retrieve Point of Contact details."""
        ...

    @abstractmethod
    def get_asn(self, asn: int) -> dict[str, Any] | None:
        """Retrieve ASN details."""
        ...

    @abstractmethod
    def sync_resources(self, rir_config: RIRConfig, resource_type: str | None = None) -> list[dict[str, Any]]:
        """Sync resources from RIR. Returns list of synced resource dicts."""
        ...

    @abstractmethod
    def update_network(self, handle: str, data: dict[str, Any]) -> dict[str, Any] | None:
        """Update a network's details (name, POC links, comments). Returns updated net dict or None."""
        ...

    @abstractmethod
    def reassign_network(self, parent_handle: str, net_data: dict[str, Any]) -> dict[str, Any] | None:
        """Reassign a subnet from parent NET. Returns ticket info dict or None."""
        ...

    @abstractmethod
    def reallocate_network(self, parent_handle: str, net_data: dict[str, Any]) -> dict[str, Any] | None:
        """Reallocate a subnet from parent NET. Returns ticket info dict or None."""
        ...

    @abstractmethod
    def remove_network(self, handle: str) -> bool:
        """Remove a reassigned/reallocated network. Returns True on success."""
        ...

    @abstractmethod
    def delete_network(self, handle: str) -> dict[str, Any] | None:
        """Delete a network. Returns ticket info dict or None."""
        ...

    @abstractmethod
    def get_customer(self, handle: str) -> dict[str, Any] | None:
        """Retrieve customer details by handle."""
        ...

    @abstractmethod
    def create_customer(self, parent_net_handle: str, data: dict[str, Any]) -> dict[str, Any] | None:
        """Create a customer under a parent NET for simple reassignment. Returns customer dict or None."""
        ...



import pytest


@pytest.mark.django_db
class TestEncryptedCharField:
    def test_encrypts_on_save_and_decrypts_on_read(self, rir_config, admin_user):
        from netbox_rir_manager.models import RIRUserKey

        key = RIRUserKey.objects.create(user=admin_user, rir_config=rir_config, api_key="my-secret-key")
        key.refresh_from_db()
        # Python attribute returns plaintext
        assert key.api_key == "my-secret-key"

    def test_raw_db_value_is_encrypted(self, rir_config, admin_user):
        from django.db import connection

        from netbox_rir_manager.models import RIRUserKey

        key = RIRUserKey.objects.create(user=admin_user, rir_config=rir_config, api_key="my-secret-key")
        with connection.cursor() as cursor:
            cursor.execute(
                "SELECT api_key FROM netbox_rir_manager_riruserkey WHERE id = %s",
                [key.pk],
            )
            raw_value = cursor.fetchone()[0]
        assert raw_value.startswith("$FERNET$")
        assert "my-secret-key" not in raw_value

    def test_handles_empty_string(self, rir_config, admin_user):
        from netbox_rir_manager.models import RIRUserKey

        key = RIRUserKey.objects.create(user=admin_user, rir_config=rir_config, api_key="")
        key.refresh_from_db()
        assert key.api_key == ""

    def test_handles_none(self, rir_config, admin_user):
        """None values pass through without encryption."""
        from netbox_rir_manager.fields import EncryptedCharField

        field = EncryptedCharField(max_length=512)
        assert field.get_prep_value(None) is None
        assert field.from_db_value(None, None, None) is None

    def test_idempotent_encryption(self, rir_config, admin_user):
        """Saving twice doesn't double-encrypt."""
        from netbox_rir_manager.models import RIRUserKey

        key = RIRUserKey.objects.create(user=admin_user, rir_config=rir_config, api_key="idempotent-key")
        key.save()  # second save
        key.refresh_from_db()
        assert key.api_key == "idempotent-key"

    def test_plaintext_migrated_on_read(self, rir_config, admin_user):
        """A plaintext value in the DB (pre-migration) decrypts correctly."""
        from django.db import connection

        from netbox_rir_manager.models import RIRUserKey

        key = RIRUserKey.objects.create(user=admin_user, rir_config=rir_config, api_key="will-be-forced-plain")
        # Simulate a pre-migration plaintext value
        with connection.cursor() as cursor:
            cursor.execute(
                "UPDATE netbox_rir_manager_riruserkey SET api_key = %s WHERE id = %s",
                ["old-plain-key", key.pk],
            )
        key.refresh_from_db()
        assert key.api_key == "old-plain-key"



from unittest.mock import MagicMock, patch

import pytest


def make_runner(cls):
    """Create a job runner with mocked job and logger (bypasses __init__)."""
    runner = cls.__new__(cls)
    runner.job = MagicMock()
    runner.job.data = {}
    runner.logger = MagicMock()
    return runner


@pytest.mark.django_db
class TestScheduledSyncJob:
    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_scheduled_sync_all_active_configs(self, mock_backend_class, rir_config, rir_user_key):
        from netbox_rir_manager.jobs import ScheduledRIRSyncJob
        from netbox_rir_manager.models import RIROrganization

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = {
            "handle": "SCHED-ARIN",
            "name": "Scheduled Org",
            "street_address": "",
            "city": "",
            "state_province": "",
            "postal_code": "",
            "country": "",
            "poc_links": [],
            "raw_data": {},
        }
        mock_backend.find_net.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(ScheduledRIRSyncJob)

        runner.run()

        assert RIROrganization.objects.filter(handle="SCHED-ARIN").exists()

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_scheduled_sync_skips_inactive_configs(self, mock_backend_class, rir_config, rir_user_key):
        from netbox_rir_manager.jobs import ScheduledRIRSyncJob

        rir_config.is_active = False
        rir_config.save()

        runner = make_runner(ScheduledRIRSyncJob)

        runner.run()

        mock_backend_class.from_rir_config.assert_not_called()

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_scheduled_sync_skips_config_without_keys(self, mock_backend_class, rir_config):
        """Config without any RIRUserKey entries should be skipped."""
        from netbox_rir_manager.jobs import ScheduledRIRSyncJob

        runner = make_runner(ScheduledRIRSyncJob)

        runner.run()

        mock_backend_class.from_rir_config.assert_not_called()

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_scheduled_sync_groups_by_synced_by_key(self, mock_backend_class, rir_config, admin_user, rir_user_key):
        """Objects with synced_by set should be refreshed using the same key."""
        from netbox_rir_manager.jobs import ScheduledRIRSyncJob
        from netbox_rir_manager.models import RIROrganization

        # Create an org that was synced_by this user key
        RIROrganization.objects.create(
            rir_config=rir_config,
            handle="GROUPED-ARIN",
            name="Grouped Org",
            synced_by=rir_user_key,
        )

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = {
            "handle": "GROUPED-ARIN",
            "name": "Grouped Org Updated",
            "street_address": "",
            "city": "",
            "state_province": "",
            "postal_code": "",
            "country": "",
            "poc_links": [],
            "raw_data": {},
        }
        mock_backend.find_net.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(ScheduledRIRSyncJob)

        runner.run()

        # Verify the backend was created with the correct key
        mock_backend_class.from_rir_config.assert_called_once_with(rir_config, api_key=rir_user_key.api_key)


@pytest.mark.django_db
class TestRIRSyncJob:
    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_creates_org_and_log(self, mock_backend_class, rir_config):
        from netbox_rir_manager.jobs import sync_rir_config
        from netbox_rir_manager.models import RIROrganization, RIRSyncLog

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = {
            "handle": "TESTORG-ARIN",
            "name": "Test Org",
            "street_address": "",
            "city": "Anytown",
            "state_province": "VA",
            "postal_code": "12345",
            "country": "US",
            "poc_links": [],
            "raw_data": {},
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        logs, agg_nets = sync_rir_config(rir_config, api_key="test-key", resource_types=["organizations"])

        assert len(logs) == 1
        assert logs[0].status == "success"
        assert agg_nets == []
        assert RIROrganization.objects.filter(handle="TESTORG-ARIN").exists()
        assert RIRSyncLog.objects.filter(rir_config=rir_config).count() == 1

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_error_creates_error_log(self, mock_backend_class, rir_config):
        from netbox_rir_manager.jobs import sync_rir_config
        from netbox_rir_manager.models import RIRSyncLog

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        logs, agg_nets = sync_rir_config(rir_config, api_key="test-key", resource_types=["organizations"])

        assert len(logs) == 1
        assert logs[0].status == "error"
        assert agg_nets == []
        assert RIRSyncLog.objects.filter(rir_config=rir_config, status="error").count() == 1

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_updates_last_sync(self, mock_backend_class, rir_config):
        from netbox_rir_manager.jobs import sync_rir_config

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        assert rir_config.last_sync is None
        sync_rir_config(rir_config, api_key="test-key", resource_types=["organizations"])

        rir_config.refresh_from_db()
        assert rir_config.last_sync is not None

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_contacts_from_org(self, mock_backend_class, rir_config):
        from netbox_rir_manager.jobs import sync_rir_config
        from netbox_rir_manager.models import RIRContact, RIROrganization

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = {
            "handle": "TESTORG-ARIN",
            "name": "Test Org",
            "street_address": "",
            "city": "Anytown",
            "state_province": "VA",
            "postal_code": "12345",
            "country": "US",
            "poc_links": [{"handle": "JD123-ARIN", "function": "AD"}],
            "raw_data": {},
        }
        mock_backend.get_poc.return_value = {
            "handle": "JD123-ARIN",
            "contact_type": "PERSON",
            "first_name": "John",
            "last_name": "Doe",
            "company_name": "Test Org",
            "email": "john@example.com",
            "phone": "",
            "raw_data": {},
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        sync_rir_config(rir_config, api_key="test-key", resource_types=["organizations", "contacts"])

        assert RIROrganization.objects.filter(handle="TESTORG-ARIN").exists()
        assert RIRContact.objects.filter(handle="JD123-ARIN").exists()
        contact = RIRContact.objects.get(handle="JD123-ARIN")
        assert contact.first_name == "John"
        assert contact.last_name == "Doe"

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_networks_from_ipam(self, mock_backend_class, rir_config, rir):
        from ipam.models import Aggregate

        from netbox_rir_manager.jobs import sync_rir_config
        from netbox_rir_manager.models import RIRNetwork

        Aggregate.objects.create(prefix="192.0.2.0/24", rir=rir)

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = None  # no org to sync
        mock_backend.find_net.return_value = {
            "handle": "NET-192-0-2-0-1",
            "net_name": "EXAMPLE-NET",
            "version": 4,
            "org_handle": "",
            "parent_net_handle": "",
            "net_blocks": [
                {"start_address": "192.0.2.0", "end_address": "192.0.2.255", "cidr_length": 24, "type": "DS"}
            ],
            "raw_data": {},
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        logs, agg_nets = sync_rir_config(rir_config, api_key="test-key", resource_types=["networks"])

        net = RIRNetwork.objects.get(handle="NET-192-0-2-0-1")
        assert net.net_name == "EXAMPLE-NET"
        assert net.aggregate is not None
        assert str(net.aggregate.prefix) == "192.0.2.0/24"
        assert len(agg_nets) == 1

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_sets_synced_by(self, mock_backend_class, rir_config, rir_user_key):
        from netbox_rir_manager.jobs import sync_rir_config
        from netbox_rir_manager.models import RIROrganization

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = {
            "handle": "SYNCBY-ARIN",
            "name": "Sync By Org",
            "street_address": "",
            "city": "",
            "state_province": "",
            "postal_code": "",
            "country": "",
            "poc_links": [],
            "raw_data": {},
        }
        mock_backend.find_net.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        sync_rir_config(rir_config, api_key="test-key", user_key=rir_user_key)

        org = RIROrganization.objects.get(handle="SYNCBY-ARIN")
        assert org.synced_by == rir_user_key

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_job_runner(self, mock_backend_class, rir_config, admin_user):
        from netbox_rir_manager.jobs import SyncRIRConfigJob
        from netbox_rir_manager.models import RIRUserKey

        RIRUserKey.objects.create(user=admin_user, rir_config=rir_config, api_key="job-key")

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = None
        mock_backend.find_net.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(SyncRIRConfigJob)
        runner.job.object_id = rir_config.pk

        runner.run(user_id=admin_user.pk)

        # Verify the job data was updated
        assert "rir_config" in runner.job.data

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_returns_agg_nets(self, mock_backend_class, rir_config, rir):
        """sync_rir_config returns (logs, agg_nets) with aggregate/network pairs."""
        from ipam.models import Aggregate

        from netbox_rir_manager.jobs import sync_rir_config
        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="10.0.0.0/8", rir=rir)

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = None
        mock_backend.find_net.return_value = {
            "handle": "NET-10-0-0-0-1",
            "net_name": "BIG-NET",
            "version": 4,
            "org_handle": "",
            "parent_net_handle": "",
            "net_blocks": [],
            "raw_data": {},
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        logs, agg_nets = sync_rir_config(rir_config, api_key="test-key", resource_types=["networks"])

        assert len(logs) == 1
        assert len(agg_nets) == 1
        returned_agg, returned_net = agg_nets[0]
        assert returned_agg.pk == agg.pk
        assert isinstance(returned_net, RIRNetwork)
        assert returned_net.handle == "NET-10-0-0-0-1"

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_network_with_customer_handle(self, mock_backend_class, rir_config, rir):
        """When a network has a customer_handle, the customer is fetched and persisted."""
        from ipam.models import Aggregate

        from netbox_rir_manager.jobs import sync_rir_config
        from netbox_rir_manager.models import RIRCustomer, RIRNetwork

        Aggregate.objects.create(prefix="192.0.2.0/24", rir=rir)

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = None
        mock_backend.find_net.return_value = {
            "handle": "NET-192-0-2-0-1",
            "net_name": "CUSTOMER-NET",
            "version": 4,
            "customer_handle": "C07654321",
            "org_handle": None,
            "parent_net_handle": "",
            "net_blocks": [],
            "raw_data": {},
        }
        mock_backend.get_customer.return_value = {
            "handle": "C07654321",
            "customer_name": "Acme Corp",
            "street_address": "123 Main St",
            "city": "Anytown",
            "state_province": "VA",
            "postal_code": "12345",
            "country": "US",
            "registration_date": "2024-01-15",
            "raw_data": {},
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        logs, agg_nets = sync_rir_config(rir_config, api_key="test-key", resource_types=["networks"])

        mock_backend.get_customer.assert_called_once_with("C07654321")
        assert RIRCustomer.objects.filter(handle="C07654321").exists()
        customer = RIRCustomer.objects.get(handle="C07654321")
        assert customer.customer_name == "Acme Corp"
        assert customer.address is not None
        assert customer.address.city == "Anytown"
        net = RIRNetwork.objects.get(handle="NET-192-0-2-0-1")
        assert customer.network == net

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_network_without_customer_skips(self, mock_backend_class, rir_config, rir):
        """Networks with org_handle (no customer_handle) should not attempt customer fetch."""
        from ipam.models import Aggregate

        from netbox_rir_manager.jobs import sync_rir_config

        Aggregate.objects.create(prefix="192.0.2.0/24", rir=rir)

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = None
        mock_backend.find_net.return_value = {
            "handle": "NET-192-0-2-0-1",
            "net_name": "ORG-NET",
            "version": 4,
            "org_handle": "TESTORG-ARIN",
            "parent_net_handle": "",
            "net_blocks": [],
            "raw_data": {},
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        sync_rir_config(rir_config, api_key="test-key", resource_types=["networks"])

        mock_backend.get_customer.assert_not_called()


@pytest.mark.django_db
class TestSyncPrefixesJob:
    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_discovers_child_prefixes(self, mock_backend_class, rir_config, rir_user_key, rir):
        """SyncPrefixesJob discovers and syncs child prefix networks."""
        from ipam.models import Aggregate, Prefix

        from netbox_rir_manager.jobs import SyncPrefixesJob
        from netbox_rir_manager.models import RIRNetwork, RIRSyncLog

        agg = Aggregate.objects.create(prefix="10.0.0.0/20", rir=rir)
        parent_net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-PARENT-20",
            net_name="PARENT-NET",
            aggregate=agg,
        )
        Prefix.objects.create(prefix="10.0.1.0/24")
        Prefix.objects.create(prefix="10.0.2.0/24")

        call_count = 0

        def find_net_side_effect(start, end):
            nonlocal call_count
            call_count += 1
            if start == "10.0.1.0":
                return {
                    "handle": "NET-CHILD-1",
                    "net_name": "CHILD-NET-1",
                    "version": 4,
                    "org_handle": "",
                    "parent_net_handle": "NET-PARENT-20",
                    "net_blocks": [],
                    "raw_data": {},
                }
            elif start == "10.0.2.0":
                # Returns parent handle -- should be skipped
                return {
                    "handle": "NET-PARENT-20",
                    "net_name": "PARENT-NET",
                    "version": 4,
                    "org_handle": "",
                    "parent_net_handle": "",
                    "net_blocks": [],
                    "raw_data": {},
                }
            return None

        mock_backend = MagicMock()
        mock_backend.find_net.side_effect = find_net_side_effect
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(SyncPrefixesJob)

        runner.run(
            aggregate_id=agg.pk,
            parent_handle=parent_net.handle,
            user_key_id=rir_user_key.pk,
        )

        # Child net should be created
        assert RIRNetwork.objects.filter(handle="NET-CHILD-1").exists()
        # Parent-returning prefix should be skipped -- no duplicate
        assert RIRNetwork.objects.filter(handle="NET-PARENT-20").count() == 1
        # Sync log for the child
        assert RIRSyncLog.objects.filter(object_handle="NET-CHILD-1").exists()

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_discovers_customer_for_child_prefix(self, mock_backend_class, rir_config, rir_user_key, rir):
        """SyncPrefixesJob fetches and persists customers for child networks with customer_handle."""
        from ipam.models import Aggregate, Prefix

        from netbox_rir_manager.jobs import SyncPrefixesJob
        from netbox_rir_manager.models import RIRCustomer, RIRNetwork

        agg = Aggregate.objects.create(prefix="10.0.0.0/20", rir=rir)
        parent_net = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-PARENT-20",
            net_name="PARENT-NET",
            aggregate=agg,
        )
        Prefix.objects.create(prefix="10.0.1.0/24")

        mock_backend = MagicMock()
        mock_backend.find_net.return_value = {
            "handle": "NET-CHILD-1",
            "net_name": "CHILD-NET-1",
            "version": 4,
            "customer_handle": "C09999999",
            "org_handle": None,
            "parent_net_handle": "NET-PARENT-20",
            "net_blocks": [],
            "raw_data": {},
        }
        mock_backend.get_customer.return_value = {
            "handle": "C09999999",
            "customer_name": "Child Customer",
            "street_address": "",
            "city": "Springfield",
            "state_province": "IL",
            "postal_code": "62701",
            "country": "US",
            "registration_date": "2024-06-01",
            "raw_data": {},
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(SyncPrefixesJob)
        runner.run(
            aggregate_id=agg.pk,
            parent_handle=parent_net.handle,
            user_key_id=rir_user_key.pk,
        )

        mock_backend.get_customer.assert_called_once_with("C09999999")
        assert RIRCustomer.objects.filter(handle="C09999999").exists()
        customer = RIRCustomer.objects.get(handle="C09999999")
        assert customer.customer_name == "Child Customer"
        assert customer.network == RIRNetwork.objects.get(handle="NET-CHILD-1")

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_logs_progress(self, mock_backend_class, rir_config, rir_user_key, rir):
        """SyncPrefixesJob logs progress via self.logger."""
        from ipam.models import Aggregate, Prefix

        from netbox_rir_manager.jobs import SyncPrefixesJob
        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="10.0.0.0/20", rir=rir)
        RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-PARENT-20",
            net_name="PARENT-NET",
            aggregate=agg,
        )
        Prefix.objects.create(prefix="10.0.1.0/24")

        mock_backend = MagicMock()
        mock_backend.find_net.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(SyncPrefixesJob)

        runner.run(
            aggregate_id=agg.pk,
            parent_handle="NET-PARENT-20",
            user_key_id=rir_user_key.pk,
        )

        # Should have logged the scanning message
        runner.logger.info.assert_called()
        first_info_call = runner.logger.info.call_args_list[0]
        assert "Scanning" in first_info_call[0][0]


@pytest.mark.django_db
class TestReassignJobPreFlight:
    """Tests for the pre-flight check in ReassignJob that detects existing ARIN reassignments."""

    @pytest.fixture
    def reassign_setup(self, rir_config, rir_user_key, rir, rir_organization):
        """Set up common objects for reassign tests.

        Creates a parent aggregate/network and a child prefix with a tenant
        linked to an RIROrganization (for the detailed reassignment path).
        """
        from ipam.models import Aggregate, Prefix
        from tenancy.models import Tenant

        from netbox_rir_manager.models import RIRNetwork

        agg = Aggregate.objects.create(prefix="10.0.0.0/20", rir=rir)
        parent = RIRNetwork.objects.create(
            rir_config=rir_config,
            handle="NET-PARENT-20",
            net_name="PARENT-NET",
            aggregate=agg,
            auto_reassign=True,
        )
        tenant = Tenant.objects.create(name="Test Tenant", slug="test-tenant")
        rir_organization.tenant = tenant
        rir_organization.save()
        pfx = Prefix.objects.create(prefix="10.0.1.0/29", tenant=tenant)
        return {"agg": agg, "parent": parent, "prefix": pfx}

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_preflight_syncs_existing_reassignment(self, mock_backend_class, reassign_setup, rir_config, rir_user_key):
        """When ARIN returns a different net than parent, job syncs and returns early."""
        from netbox_rir_manager.jobs import ReassignJob
        from netbox_rir_manager.models import RIRNetwork, RIRSyncLog

        mock_backend = MagicMock()
        mock_backend.find_net.return_value = {
            "handle": "NET-INTERMEDIATE-24",
            "net_name": "INTERMEDIATE-NET",
            "net_type": "RS",
            "org_handle": "",
        }
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(ReassignJob)

        runner.run(
            prefix_id=reassign_setup["prefix"].pk,
            user_key_id=rir_user_key.pk,
        )

        # Should have synced the intermediate net, not called reassign
        assert runner.job.data["status"] == "synced"
        assert "NET-INTERMEDIATE-24" in runner.job.data["message"]
        mock_backend.reassign_network.assert_not_called()

        # RIRNetwork should be created for the intermediate net
        assert RIRNetwork.objects.filter(handle="NET-INTERMEDIATE-24").exists()
        net = RIRNetwork.objects.get(handle="NET-INTERMEDIATE-24")
        assert net.prefix == reassign_setup["prefix"]

        # Sync log should record skipped
        log = RIRSyncLog.objects.filter(object_handle="NET-INTERMEDIATE-24", status="skipped").first()
        assert log is not None
        assert "already reassigned" in log.message

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_preflight_proceeds_when_arin_returns_parent(
        self, mock_backend_class, reassign_setup, rir_config, rir_user_key
    ):
        """When ARIN returns the parent handle, pre-flight passes and reassignment proceeds."""
        from netbox_rir_manager.jobs import ReassignJob

        mock_backend = MagicMock()
        # find_net returns parent handle -- no intermediate reassignment
        mock_backend.find_net.return_value = {
            "handle": "NET-PARENT-20",
            "net_name": "PARENT-NET",
        }
        # Reassignment itself fails (we just need to verify it was attempted)
        mock_backend.reassign_network.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(ReassignJob)

        runner.run(
            prefix_id=reassign_setup["prefix"].pk,
            user_key_id=rir_user_key.pk,
        )

        # Pre-flight should NOT have short-circuited -- reassign was attempted
        mock_backend.reassign_network.assert_called_once()

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_preflight_proceeds_when_arin_returns_none(
        self, mock_backend_class, reassign_setup, rir_config, rir_user_key
    ):
        """When ARIN returns None for find_net, pre-flight passes and reassignment proceeds."""
        from netbox_rir_manager.jobs import ReassignJob

        mock_backend = MagicMock()
        mock_backend.find_net.return_value = None
        mock_backend.reassign_network.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(ReassignJob)

        runner.run(
            prefix_id=reassign_setup["prefix"].pk,
            user_key_id=rir_user_key.pk,
        )

        # Pre-flight should NOT have short-circuited
        mock_backend.reassign_network.assert_called_once()


@pytest.mark.django_db
class TestRemoveNetworkJob:
    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_remove_success(self, mock_backend_class, rir_config, rir_user_key):
        from netbox_rir_manager.jobs import RemoveNetworkJob
        from netbox_rir_manager.models import RIRNetwork, RIRSyncLog

        net = RIRNetwork.objects.create(rir_config=rir_config, handle="NET-REMOVE-1", net_name="REMOVE-NET")

        mock_backend = MagicMock()
        mock_backend.remove_network.return_value = True
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(RemoveNetworkJob)

        runner.run(network_id=net.pk, user_key_id=rir_user_key.pk)

        mock_backend.remove_network.assert_called_once_with("NET-REMOVE-1")
        assert runner.job.data["status"] == "success"

        log = RIRSyncLog.objects.get(object_handle="NET-REMOVE-1", operation="remove")
        assert log.status == "success"
        assert "Removed" in log.message

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_remove_failure(self, mock_backend_class, rir_config, rir_user_key):
        from netbox_rir_manager.jobs import RemoveNetworkJob
        from netbox_rir_manager.models import RIRNetwork, RIRSyncLog

        net = RIRNetwork.objects.create(rir_config=rir_config, handle="NET-RMFAIL-1", net_name="FAIL-NET")

        mock_backend = MagicMock()
        mock_backend.remove_network.return_value = False
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(RemoveNetworkJob)

        runner.run(network_id=net.pk, user_key_id=rir_user_key.pk)

        assert runner.job.data["status"] == "error"
        assert runner.job.data["message"] == "ARIN removal failed"

        log = RIRSyncLog.objects.get(object_handle="NET-RMFAIL-1", operation="remove")
        assert log.status == "error"


@pytest.mark.django_db
class TestChangelogEntries:
    """Verify that sync operations create ObjectChange changelog entries."""

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_creates_changelog_for_org(self, mock_backend_class, rir_config, admin_user, rir_user_key):
        """SyncRIRConfigJob creates ObjectChange entries when a real user is present."""
        from core.models import ObjectChange

        from netbox_rir_manager.jobs import SyncRIRConfigJob
        from netbox_rir_manager.models import RIROrganization

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = {
            "handle": "CHANGELOG-ARIN",
            "name": "Changelog Org",
            "street_address": "",
            "city": "",
            "state_province": "",
            "postal_code": "",
            "country": "",
            "poc_links": [],
            "raw_data": {},
        }
        mock_backend.find_net.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(SyncRIRConfigJob)
        runner.job.object_id = rir_config.pk
        runner.job.user = admin_user

        runner.run(user_id=admin_user.pk)

        assert RIROrganization.objects.filter(handle="CHANGELOG-ARIN").exists()
        org = RIROrganization.objects.get(handle="CHANGELOG-ARIN")
        changes = ObjectChange.objects.filter(
            changed_object_id=org.pk,
            changed_object_type__model="rirorganization",
        )
        assert changes.exists()
        assert changes.first().user == admin_user

    @patch("netbox_rir_manager.jobs.ARINBackend")
    def test_sync_no_changelog_without_user(self, mock_backend_class, rir_config, rir_user_key):
        """When job.user is a MagicMock (no real user), no ObjectChange is created."""
        from core.models import ObjectChange

        from netbox_rir_manager.jobs import SyncRIRConfigJob

        mock_backend = MagicMock()
        mock_backend.get_organization.return_value = {
            "handle": "NOLOG-ARIN",
            "name": "No Log Org",
            "street_address": "",
            "city": "",
            "state_province": "",
            "postal_code": "",
            "country": "",
            "poc_links": [],
            "raw_data": {},
        }
        mock_backend.find_net.return_value = None
        mock_backend_class.from_rir_config.return_value = mock_backend

        runner = make_runner(SyncRIRConfigJob)
        runner.job.object_id = rir_config.pk
        # runner.job.user is MagicMock (default from make_runner)

        initial_count = ObjectChange.objects.count()
        runner.run(user_id=rir_user_key.user_id)

        # No ObjectChange should be created since user is not a real User
        assert ObjectChange.objects.count() == initial_count



import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcim", "0001_initial"),
        ("extras", "0001_initial"),
        ("tenancy", "0001_initial"),
        ("netbox_rir_manager", "0010_add_rircontact_address_fields"),
    ]

    operations = [
        # Create RIRSiteAddress model
        migrations.CreateModel(
            name="RIRSiteAddress",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("street_address", models.TextField(blank=True, default="")),
                ("city", models.CharField(blank=True, default="", max_length=100)),
                (
                    "state_province",
                    models.CharField(
                        blank=True, default="", help_text="ISO-3166-2 subdivision code (e.g. NY, QC)", max_length=10
                    ),
                ),
                ("postal_code", models.CharField(blank=True, default="", max_length=20)),
                (
                    "country",
                    models.CharField(blank=True, default="", help_text="ISO-3166-1 alpha-2 country code", max_length=2),
                ),
                (
                    "raw_geocode",
                    models.JSONField(blank=True, default=dict, help_text="Full geocoder response for debugging"),
                ),
                (
                    "auto_resolved",
                    models.BooleanField(default=False, help_text="True if geocoded, False if manually entered"),
                ),
                ("last_resolved", models.DateTimeField(blank=True, null=True)),
                (
                    "site",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE, related_name="rir_address", to="dcim.site"
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
            ],
            options={
                "verbose_name": "RIR site address",
                "verbose_name_plural": "RIR site addresses",
                "ordering": ["site__name"],
            },
        ),
        # Add tenant FK to RIROrganization
        migrations.AddField(
            model_name="rirorganization",
            name="tenant",
            field=models.ForeignKey(
                blank=True,
                help_text="Link to a NetBox Tenant for automatic detailed reassignment",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="rir_organizations",
                to="tenancy.tenant",
            ),
        ),
        # Add auto_reassign to RIRNetwork
        migrations.AddField(
            model_name="rirnetwork",
            name="auto_reassign",
            field=models.BooleanField(
                default=False,
                help_text="Automatically reassign child prefixes at ARIN when they get a Site and Tenant",
            ),
        ),
    ]



from netbox.api.routers import NetBoxRouter

from netbox_rir_manager.api import views

router = NetBoxRouter()
router.register("configs", views.RIRConfigViewSet)
router.register("organizations", views.RIROrganizationViewSet)
router.register("contacts", views.RIRContactViewSet)
router.register("customers", views.RIRCustomerViewSet)
router.register("networks", views.RIRNetworkViewSet)
router.register("addresses", views.RIRAddressViewSet)
router.register("sync-logs", views.RIRSyncLogViewSet)
router.register("tickets", views.RIRTicketViewSet)
router.register("user-keys", views.RIRUserKeyViewSet)

urlpatterns = router.urls



# Generated by Django 5.2.9 on 2026-02-04 18:46

import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("extras", "0134_owner"),
        ("ipam", "0086_gfk_indexes"),
    ]

    operations = [
        migrations.CreateModel(
            name="RIRAccount",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("name", models.CharField(max_length=100)),
                ("api_key", models.CharField(max_length=255)),
                ("api_url", models.URLField(blank=True, default="")),
                ("org_handle", models.CharField(blank=True, default="", max_length=50)),
                ("is_active", models.BooleanField(default=True)),
                ("last_sync", models.DateTimeField(blank=True, null=True)),
                (
                    "rir",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, related_name="rir_accounts", to="ipam.rir"
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
            ],
            options={
                "ordering": ["rir", "name"],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name="RIROrganization",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("handle", models.CharField(max_length=50, unique=True)),
                ("name", models.CharField(max_length=255)),
                ("street_address", models.TextField(blank=True, default="")),
                ("city", models.CharField(blank=True, default="", max_length=100)),
                ("state_province", models.CharField(blank=True, default="", max_length=100)),
                ("postal_code", models.CharField(blank=True, default="", max_length=20)),
                ("country", models.CharField(blank=True, default="", max_length=2)),
                ("raw_data", models.JSONField(blank=True, default=dict)),
                ("last_synced", models.DateTimeField(blank=True, null=True)),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="organizations",
                        to="netbox_rir_manager.riraccount",
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
            ],
            options={
                "ordering": ["handle"],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name="RIRNetwork",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("handle", models.CharField(max_length=50, unique=True)),
                ("net_name", models.CharField(max_length=100)),
                ("raw_data", models.JSONField(blank=True, default=dict)),
                ("last_synced", models.DateTimeField(blank=True, null=True)),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="networks",
                        to="netbox_rir_manager.riraccount",
                    ),
                ),
                (
                    "aggregate",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="rir_networks",
                        to="ipam.aggregate",
                    ),
                ),
                (
                    "prefix",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="rir_networks",
                        to="ipam.prefix",
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
                (
                    "organization",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="networks",
                        to="netbox_rir_manager.rirorganization",
                    ),
                ),
            ],
            options={
                "ordering": ["handle"],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name="RIRContact",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("handle", models.CharField(max_length=50, unique=True)),
                ("contact_type", models.CharField(max_length=20)),
                ("first_name", models.CharField(blank=True, default="", max_length=100)),
                ("last_name", models.CharField(max_length=100)),
                ("company_name", models.CharField(blank=True, default="", max_length=255)),
                ("email", models.EmailField(blank=True, default="", max_length=254)),
                ("phone", models.CharField(blank=True, default="", max_length=50)),
                ("raw_data", models.JSONField(blank=True, default=dict)),
                ("last_synced", models.DateTimeField(blank=True, null=True)),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="contacts",
                        to="netbox_rir_manager.riraccount",
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
                (
                    "organization",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="contacts",
                        to="netbox_rir_manager.rirorganization",
                    ),
                ),
            ],
            options={
                "ordering": ["handle"],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name="RIRSyncLog",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ("operation", models.CharField(max_length=50)),
                ("object_type", models.CharField(max_length=50)),
                ("object_handle", models.CharField(max_length=100)),
                ("status", models.CharField(max_length=20)),
                ("message", models.TextField(blank=True, default="")),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sync_logs",
                        to="netbox_rir_manager.riraccount",
                    ),
                ),
                ("tags", taggit.managers.TaggableManager(through="extras.TaggedItem", to="extras.Tag")),
            ],
            options={
                "ordering": ["-created"],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.AddConstraint(
            model_name="riraccount",
            constraint=models.UniqueConstraint(fields=("rir", "name"), name="unique_rir_account_name"),
        ),
    ]



import pytest


@pytest.mark.django_db
class TestRIRConfigFilterSet:
    def test_search_by_name(self, rir_config):
        from netbox_rir_manager.filtersets import RIRConfigFilterSet
        from netbox_rir_manager.models import RIRConfig

        qs = RIRConfig.objects.all()
        fs = RIRConfigFilterSet({"q": "ARIN"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_search_no_match(self, rir_config):
        from netbox_rir_manager.filtersets import RIRConfigFilterSet
        from netbox_rir_manager.models import RIRConfig

        qs = RIRConfig.objects.all()
        fs = RIRConfigFilterSet({"q": "nonexistent"}, queryset=qs)
        assert fs.qs.count() == 0

    def test_filter_by_active(self, rir, rir_config):
        from netbox_rir_manager.filtersets import RIRConfigFilterSet
        from netbox_rir_manager.models import RIRConfig

        RIRConfig.objects.create(rir=rir, name="Inactive", is_active=False)
        qs = RIRConfig.objects.all()

        fs = RIRConfigFilterSet({"is_active": True}, queryset=qs)
        assert fs.qs.count() == 1
        assert fs.qs.first().name == rir_config.name

        fs = RIRConfigFilterSet({"is_active": False}, queryset=qs)
        assert fs.qs.count() == 1
        assert fs.qs.first().name == "Inactive"


@pytest.mark.django_db
class TestRIROrganizationFilterSet:
    def test_search_by_handle(self, rir_organization):
        from netbox_rir_manager.filtersets import RIROrganizationFilterSet
        from netbox_rir_manager.models import RIROrganization

        qs = RIROrganization.objects.all()
        fs = RIROrganizationFilterSet({"q": "TESTORG"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_search_by_name(self, rir_organization):
        from netbox_rir_manager.filtersets import RIROrganizationFilterSet
        from netbox_rir_manager.models import RIROrganization

        qs = RIROrganization.objects.all()
        fs = RIROrganizationFilterSet({"q": "Test Organization"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_filter_by_rir_config(self, rir_organization, rir_config):
        from netbox_rir_manager.filtersets import RIROrganizationFilterSet
        from netbox_rir_manager.models import RIROrganization

        qs = RIROrganization.objects.all()
        fs = RIROrganizationFilterSet({"rir_config_id": [rir_config.pk]}, queryset=qs)
        assert fs.qs.count() == 1


@pytest.mark.django_db
class TestRIRContactFilterSet:
    def test_search_by_handle(self, rir_contact):
        from netbox_rir_manager.filtersets import RIRContactFilterSet
        from netbox_rir_manager.models import RIRContact

        qs = RIRContact.objects.all()
        fs = RIRContactFilterSet({"q": "JD123"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_search_by_last_name(self, rir_contact):
        from netbox_rir_manager.filtersets import RIRContactFilterSet
        from netbox_rir_manager.models import RIRContact

        qs = RIRContact.objects.all()
        fs = RIRContactFilterSet({"q": "Doe"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_filter_by_contact_type(self, rir_contact):
        from netbox_rir_manager.filtersets import RIRContactFilterSet
        from netbox_rir_manager.models import RIRContact

        qs = RIRContact.objects.all()
        fs = RIRContactFilterSet({"contact_type": "PERSON"}, queryset=qs)
        assert fs.qs.count() == 1

        fs = RIRContactFilterSet({"contact_type": "ROLE"}, queryset=qs)
        assert fs.qs.count() == 0


@pytest.mark.django_db
class TestRIRNetworkFilterSet:
    def test_search_by_handle(self, rir_network):
        from netbox_rir_manager.filtersets import RIRNetworkFilterSet
        from netbox_rir_manager.models import RIRNetwork

        qs = RIRNetwork.objects.all()
        fs = RIRNetworkFilterSet({"q": "NET-192"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_search_by_net_name(self, rir_network):
        from netbox_rir_manager.filtersets import RIRNetworkFilterSet
        from netbox_rir_manager.models import RIRNetwork

        qs = RIRNetwork.objects.all()
        fs = RIRNetworkFilterSet({"q": "EXAMPLE"}, queryset=qs)
        assert fs.qs.count() == 1


@pytest.mark.django_db
class TestRIRSyncLogFilterSet:
    def test_search_by_handle(self, rir_config):
        from netbox_rir_manager.filtersets import RIRSyncLogFilterSet
        from netbox_rir_manager.models import RIRSyncLog

        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="SEARCHME", status="success"
        )
        qs = RIRSyncLog.objects.all()
        fs = RIRSyncLogFilterSet({"q": "SEARCHME"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_filter_by_operation(self, rir_config):
        from netbox_rir_manager.filtersets import RIRSyncLogFilterSet
        from netbox_rir_manager.models import RIRSyncLog

        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="A", status="success"
        )
        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="create", object_type="org", object_handle="B", status="success"
        )
        qs = RIRSyncLog.objects.all()
        fs = RIRSyncLogFilterSet({"operation": "sync"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_filter_by_status(self, rir_config):
        from netbox_rir_manager.filtersets import RIRSyncLogFilterSet
        from netbox_rir_manager.models import RIRSyncLog

        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="A", status="success"
        )
        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="B", status="error"
        )
        qs = RIRSyncLog.objects.all()
        fs = RIRSyncLogFilterSet({"status": "error"}, queryset=qs)
        assert fs.qs.count() == 1


@pytest.mark.django_db
class TestRIRTicketFilterSet:
    def test_search_by_ticket_number(self, rir_ticket):
        from netbox_rir_manager.filtersets import RIRTicketFilterSet
        from netbox_rir_manager.models import RIRTicket

        qs = RIRTicket.objects.all()
        fs = RIRTicketFilterSet({"q": "TKT-TEST"}, queryset=qs)
        assert fs.qs.count() == 1

    def test_search_no_match(self, rir_ticket):
        from netbox_rir_manager.filtersets import RIRTicketFilterSet
        from netbox_rir_manager.models import RIRTicket

        qs = RIRTicket.objects.all()
        fs = RIRTicketFilterSet({"q": "nonexistent"}, queryset=qs)
        assert fs.qs.count() == 0

    def test_filter_by_status(self, rir_ticket):
        from netbox_rir_manager.filtersets import RIRTicketFilterSet
        from netbox_rir_manager.models import RIRTicket

        qs = RIRTicket.objects.all()
        fs = RIRTicketFilterSet({"status": "pending_review"}, queryset=qs)
        assert fs.qs.count() == 1

        fs = RIRTicketFilterSet({"status": "resolved"}, queryset=qs)
        assert fs.qs.count() == 0

    def test_filter_by_ticket_type(self, rir_ticket):
        from netbox_rir_manager.filtersets import RIRTicketFilterSet
        from netbox_rir_manager.models import RIRTicket

        qs = RIRTicket.objects.all()
        fs = RIRTicketFilterSet({"ticket_type": "IPV4_SIMPLE_REASSIGN"}, queryset=qs)
        assert fs.qs.count() == 1

        fs = RIRTicketFilterSet({"ticket_type": "IPV6_REALLOCATE"}, queryset=qs)
        assert fs.qs.count() == 0

    def test_filter_by_rir_config(self, rir_ticket, rir_config):
        from netbox_rir_manager.filtersets import RIRTicketFilterSet
        from netbox_rir_manager.models import RIRTicket

        qs = RIRTicket.objects.all()
        fs = RIRTicketFilterSet({"rir_config_id": [rir_config.pk]}, queryset=qs)
        assert fs.qs.count() == 1



from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel

from netbox_rir_manager.choices import TicketResolutionChoices, TicketStatusChoices, TicketTypeChoices


class RIRTicket(NetBoxModel):
    """Tracks ARIN ticket requests from write operations."""

    rir_config = models.ForeignKey(
        "netbox_rir_manager.RIRConfig",
        on_delete=models.CASCADE,
        related_name="tickets",
        verbose_name="RIR config",
    )
    ticket_number = models.CharField(max_length=50, unique=True)
    ticket_type = models.CharField(max_length=50, choices=TicketTypeChoices)
    status = models.CharField(max_length=30, choices=TicketStatusChoices)
    resolution = models.CharField(max_length=30, choices=TicketResolutionChoices, blank=True, default="")
    network = models.ForeignKey(
        "netbox_rir_manager.RIRNetwork",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="tickets",
    )
    submitted_by = models.ForeignKey(
        "netbox_rir_manager.RIRUserKey",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="tickets",
    )
    created_date = models.DateTimeField()
    resolved_date = models.DateTimeField(null=True, blank=True)
    raw_data = models.JSONField(default=dict, blank=True)

    class Meta:
        ordering = ["-created_date"]
        verbose_name = "RIR ticket"
        verbose_name_plural = "RIR tickets"

    def __str__(self):
        return f"Ticket {self.ticket_number}"

    def get_absolute_url(self):
        return reverse("plugins:netbox_rir_manager:rirticket", args=[self.pk])



# Generated by Django 5.2.9 on 2026-02-13 03:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("netbox_rir_manager", "0011_rirsiteaddress_rirorganization_tenant_rirnetwork_auto_reassign"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rircontact",
            name="last_name",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
    ]



from netbox.api.serializers import NetBoxModelSerializer
from rest_framework import serializers

from netbox_rir_manager.models import (
    RIRAddress,
    RIRConfig,
    RIRContact,
    RIRCustomer,
    RIRNetwork,
    RIROrganization,
    RIRSyncLog,
    RIRTicket,
    RIRUserKey,
)


class RIRConfigSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:rirconfig-detail")

    class Meta:
        model = RIRConfig
        fields = (
            "id",
            "url",
            "display",
            "rir",
            "name",
            "api_url",
            "org_handle",
            "is_active",
            "last_sync",
            "tags",
            "created",
            "last_updated",
        )


class RIROrganizationSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:rirorganization-detail")

    class Meta:
        model = RIROrganization
        fields = (
            "id",
            "url",
            "display",
            "rir_config",
            "handle",
            "name",
            "tenant",
            "address",
            "raw_data",
            "last_synced",
            "tags",
            "created",
            "last_updated",
        )


class RIRContactSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:rircontact-detail")

    class Meta:
        model = RIRContact
        fields = (
            "id",
            "url",
            "display",
            "rir_config",
            "handle",
            "contact_type",
            "first_name",
            "last_name",
            "company_name",
            "email",
            "phone",
            "address",
            "organization",
            "contact",
            "raw_data",
            "last_synced",
            "tags",
            "created",
            "last_updated",
        )


class RIRCustomerSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:rircustomer-detail")

    class Meta:
        model = RIRCustomer
        fields = (
            "id",
            "url",
            "display",
            "rir_config",
            "handle",
            "customer_name",
            "address",
            "network",
            "tenant",
            "raw_data",
            "created_date",
            "tags",
            "created",
            "last_updated",
        )


class RIRNetworkSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:rirnetwork-detail")

    class Meta:
        model = RIRNetwork
        fields = (
            "id",
            "url",
            "display",
            "rir_config",
            "handle",
            "net_name",
            "net_type",
            "organization",
            "auto_reassign",
            "aggregate",
            "prefix",
            "raw_data",
            "last_synced",
            "tags",
            "created",
            "last_updated",
        )


class RIRSyncLogSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:rirsynclog-detail")

    class Meta:
        model = RIRSyncLog
        fields = (
            "id",
            "url",
            "display",
            "rir_config",
            "operation",
            "object_type",
            "object_handle",
            "status",
            "message",
            "tags",
            "created",
            "last_updated",
        )


class RIRTicketSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:rirticket-detail")

    class Meta:
        model = RIRTicket
        fields = (
            "id",
            "url",
            "display",
            "rir_config",
            "ticket_number",
            "ticket_type",
            "status",
            "resolution",
            "network",
            "submitted_by",
            "created_date",
            "resolved_date",
            "raw_data",
            "tags",
            "created",
            "last_updated",
        )


class RIRUserKeySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:riruserkey-detail")
    api_key = serializers.CharField(write_only=True)

    class Meta:
        model = RIRUserKey
        fields = (
            "id",
            "url",
            "display",
            "user",
            "rir_config",
            "api_key",
            "tags",
            "created",
            "last_updated",
        )


class RIRAddressSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_rir_manager-api:riraddress-detail")

    class Meta:
        model = RIRAddress
        fields = (
            "id",
            "url",
            "display",
            "site",
            "street_address",
            "city",
            "state_province",
            "postal_code",
            "country",
            "raw_geocode",
            "auto_resolved",
            "last_resolved",
            "tags",
            "created",
            "last_updated",
        )


class NetworkReassignSerializer(serializers.Serializer):
    reassignment_type = serializers.ChoiceField(choices=["simple", "detailed"])
    customer_name = serializers.CharField(max_length=255, required=False, allow_blank=True, default="")
    street_address = serializers.CharField(required=False, allow_blank=True, default="")
    city = serializers.CharField(max_length=100, required=False, allow_blank=True, default="")
    state_province = serializers.CharField(max_length=100, required=False, allow_blank=True, default="")
    postal_code = serializers.CharField(max_length=20, required=False, allow_blank=True, default="")
    country = serializers.CharField(max_length=2, required=False, allow_blank=True, default="")
    org_handle = serializers.CharField(max_length=50, required=False, allow_blank=True, default="")
    net_name = serializers.CharField(max_length=100, required=False, allow_blank=True, default="")
    start_address = serializers.IPAddressField()
    end_address = serializers.IPAddressField()

    def validate(self, data):
        rtype = data.get("reassignment_type")
        if rtype == "simple":
            for field in ("customer_name", "city", "country"):
                if not data.get(field):
                    raise serializers.ValidationError({field: "Required for simple reassignment."})
        elif rtype == "detailed" and not data.get("org_handle"):
            raise serializers.ValidationError({"org_handle": "Required for detailed reassignment."})
        return data


class NetworkReallocateSerializer(serializers.Serializer):
    org_handle = serializers.CharField(max_length=50)
    net_name = serializers.CharField(max_length=100, required=False, allow_blank=True, default="")
    start_address = serializers.IPAddressField()
    end_address = serializers.IPAddressField()



from unittest.mock import MagicMock, patch

import pytest
from django.urls import reverse
from rest_framework import status


@pytest.mark.django_db
class TestRIRConfigAPI:
    def test_list_configs(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirconfig-list")
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

    def test_get_config(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirconfig-detail", args=[rir_config.pk])
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["name"] == rir_config.name

    def test_create_config(self, admin_api_client, rir):
        url = reverse("plugins-api:netbox_rir_manager-api:rirconfig-list")
        data = {
            "rir": rir.pk,
            "name": "New API Config",
            "org_handle": "NEW-ARIN",
            "is_active": True,
        }
        response = admin_api_client.post(url, data, format="json")
        assert response.status_code == status.HTTP_201_CREATED
        assert response.json()["name"] == "New API Config"

    def test_update_config(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirconfig-detail", args=[rir_config.pk])
        response = admin_api_client.patch(url, {"name": "Updated Name"}, format="json")
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["name"] == "Updated Name"

    def test_delete_config(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirconfig-detail", args=[rir_config.pk])
        response = admin_api_client.delete(url)
        assert response.status_code == status.HTTP_204_NO_CONTENT

    def test_filter_by_active(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirconfig-list")
        response = admin_api_client.get(url, {"is_active": "true"})
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

        response = admin_api_client.get(url, {"is_active": "false"})
        assert response.json()["count"] == 0


@pytest.mark.django_db
class TestRIROrganizationAPI:
    def test_list_organizations(self, admin_api_client, rir_organization):
        url = reverse("plugins-api:netbox_rir_manager-api:rirorganization-list")
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

    def test_get_organization(self, admin_api_client, rir_organization):
        url = reverse("plugins-api:netbox_rir_manager-api:rirorganization-detail", args=[rir_organization.pk])
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["handle"] == rir_organization.handle

    def test_create_organization(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirorganization-list")
        data = {
            "rir_config": rir_config.pk,
            "handle": "NEWORG-ARIN",
            "name": "New Organization",
        }
        response = admin_api_client.post(url, data, format="json")
        assert response.status_code == status.HTTP_201_CREATED
        assert response.json()["handle"] == "NEWORG-ARIN"

    def test_filter_by_rir_config(self, admin_api_client, rir_organization, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirorganization-list")
        response = admin_api_client.get(url, {"rir_config_id": rir_config.pk})
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1


@pytest.mark.django_db
class TestRIRContactAPI:
    def test_list_contacts(self, admin_api_client, rir_contact):
        url = reverse("plugins-api:netbox_rir_manager-api:rircontact-list")
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

    def test_get_contact(self, admin_api_client, rir_contact):
        url = reverse("plugins-api:netbox_rir_manager-api:rircontact-detail", args=[rir_contact.pk])
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["handle"] == rir_contact.handle

    def test_create_contact(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rircontact-list")
        data = {
            "rir_config": rir_config.pk,
            "handle": "NEWPOC-ARIN",
            "contact_type": "ROLE",
            "last_name": "NOC",
            "email": "noc@example.com",
        }
        response = admin_api_client.post(url, data, format="json")
        assert response.status_code == status.HTTP_201_CREATED
        assert response.json()["handle"] == "NEWPOC-ARIN"

    def test_filter_by_contact_type(self, admin_api_client, rir_contact):
        url = reverse("plugins-api:netbox_rir_manager-api:rircontact-list")
        response = admin_api_client.get(url, {"contact_type": "PERSON"})
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

        response = admin_api_client.get(url, {"contact_type": "ROLE"})
        assert response.json()["count"] == 0


@pytest.mark.django_db
class TestRIRNetworkAPI:
    def test_list_networks(self, admin_api_client, rir_network):
        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-list")
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

    def test_get_network(self, admin_api_client, rir_network):
        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-detail", args=[rir_network.pk])
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["handle"] == rir_network.handle

    def test_create_network(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-list")
        data = {
            "rir_config": rir_config.pk,
            "handle": "NET-10-0-0-0-1",
            "net_name": "TEN-NET",
        }
        response = admin_api_client.post(url, data, format="json")
        assert response.status_code == status.HTTP_201_CREATED
        assert response.json()["net_name"] == "TEN-NET"


@pytest.mark.django_db
class TestRIRSyncLogAPI:
    def test_list_sync_logs(self, admin_api_client, rir_config):
        from netbox_rir_manager.models import RIRSyncLog

        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="TEST", status="success"
        )
        url = reverse("plugins-api:netbox_rir_manager-api:rirsynclog-list")
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

    def test_get_sync_log(self, admin_api_client, rir_config):
        from netbox_rir_manager.models import RIRSyncLog

        log = RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="TEST", status="success"
        )
        url = reverse("plugins-api:netbox_rir_manager-api:rirsynclog-detail", args=[log.pk])
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["status"] == "success"

    def test_filter_by_status(self, admin_api_client, rir_config):
        from netbox_rir_manager.models import RIRSyncLog

        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="A", status="success"
        )
        RIRSyncLog.objects.create(
            rir_config=rir_config, operation="sync", object_type="org", object_handle="B", status="error"
        )
        url = reverse("plugins-api:netbox_rir_manager-api:rirsynclog-list")
        response = admin_api_client.get(url, {"status": "success"})
        assert response.json()["count"] == 1
        response = admin_api_client.get(url, {"status": "error"})
        assert response.json()["count"] == 1


@pytest.mark.django_db
class TestRIRTicketAPI:
    def test_list_tickets(self, admin_api_client, rir_ticket):
        url = reverse("plugins-api:netbox_rir_manager-api:rirticket-list")
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

    def test_get_ticket(self, admin_api_client, rir_ticket):
        url = reverse("plugins-api:netbox_rir_manager-api:rirticket-detail", args=[rir_ticket.pk])
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["ticket_number"] == rir_ticket.ticket_number

    def test_create_ticket(self, admin_api_client, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:rirticket-list")
        data = {
            "rir_config": rir_config.pk,
            "ticket_number": "TKT-API-001",
            "ticket_type": "IPV4_SIMPLE_REASSIGN",
            "status": "pending_review",
            "created_date": "2024-01-01T00:00:00Z",
        }
        response = admin_api_client.post(url, data, format="json")
        assert response.status_code == status.HTTP_201_CREATED
        assert response.json()["ticket_number"] == "TKT-API-001"

    def test_filter_by_status(self, admin_api_client, rir_ticket):
        url = reverse("plugins-api:netbox_rir_manager-api:rirticket-list")
        response = admin_api_client.get(url, {"status": "pending_review"})
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

        response = admin_api_client.get(url, {"status": "resolved"})
        assert response.json()["count"] == 0


@pytest.mark.django_db
class TestRIRUserKeyAPI:
    def test_list_user_keys(self, admin_api_client, rir_user_key):
        url = reverse("plugins-api:netbox_rir_manager-api:riruserkey-list")
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json()["count"] == 1

    def test_get_user_key(self, admin_api_client, rir_user_key):
        url = reverse("plugins-api:netbox_rir_manager-api:riruserkey-detail", args=[rir_user_key.pk])
        response = admin_api_client.get(url)
        assert response.status_code == status.HTTP_200_OK
        # api_key should NOT be in response (write-only)
        assert "api_key" not in response.json()

    def test_create_user_key(self, admin_api_client, admin_user, rir_config):
        url = reverse("plugins-api:netbox_rir_manager-api:riruserkey-list")
        data = {
            "user": admin_user.pk,
            "rir_config": rir_config.pk,
            "api_key": "new-api-key",
        }
        response = admin_api_client.post(url, data, format="json")
        assert response.status_code == status.HTTP_201_CREATED


@pytest.mark.django_db
class TestAPIAuthentication:
    def test_unauthenticated_request_rejected(self):
        from rest_framework.test import APIClient

        client = APIClient()
        url = reverse("plugins-api:netbox_rir_manager-api:rirconfig-list")
        response = client.get(url)
        assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
class TestRIRNetworkAPIActions:
    @patch("netbox_rir_manager.api.views.ARINBackend")
    def test_reassign_api_simple(self, mock_backend_cls, admin_api_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.create_customer.return_value = {"handle": "C-TEST"}
        mock_backend.reassign_network.return_value = {
            "ticket_number": "TKT-API-001",
            "ticket_type": "IPV4_SIMPLE_REASSIGN",
            "ticket_status": "PENDING_REVIEW",
            "raw_data": {},
        }
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-reassign", args=[rir_network.pk])
        response = admin_api_client.post(
            url,
            {
                "reassignment_type": "simple",
                "customer_name": "Test Customer",
                "city": "Testville",
                "country": "US",
                "start_address": "10.0.0.0",
                "end_address": "10.0.0.255",
            },
            format="json",
        )
        assert response.status_code == status.HTTP_201_CREATED
        assert "ticket_number" in response.json()

    @patch("netbox_rir_manager.api.views.ARINBackend")
    def test_reassign_api_no_key(self, mock_backend_cls, admin_api_client, rir_network):
        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-reassign", args=[rir_network.pk])
        response = admin_api_client.post(
            url,
            {
                "reassignment_type": "detailed",
                "org_handle": "TARGET-ORG",
                "start_address": "10.0.0.0",
                "end_address": "10.0.0.255",
            },
            format="json",
        )
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_reassign_api_invalid_data(self, admin_api_client, rir_network, rir_user_key):
        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-reassign", args=[rir_network.pk])
        response = admin_api_client.post(
            url,
            {
                "reassignment_type": "simple",
                # Missing required fields: start_address, end_address
            },
            format="json",
        )
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    @patch("netbox_rir_manager.api.views.ARINBackend")
    def test_reallocate_api(self, mock_backend_cls, admin_api_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.reallocate_network.return_value = {
            "ticket_number": "TKT-API-REALLOC",
            "ticket_type": "IPV4_REALLOCATE",
            "ticket_status": "PENDING_REVIEW",
            "raw_data": {},
        }
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-reallocate", args=[rir_network.pk])
        response = admin_api_client.post(
            url,
            {
                "org_handle": "TARGET-ORG",
                "start_address": "10.0.0.0",
                "end_address": "10.0.0.255",
            },
            format="json",
        )
        assert response.status_code == status.HTTP_201_CREATED

    @patch("netbox_rir_manager.api.views.ARINBackend")
    def test_remove_api(self, mock_backend_cls, admin_api_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.remove_network.return_value = True
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-remove-net", args=[rir_network.pk])
        response = admin_api_client.post(url, format="json")
        assert response.status_code == status.HTTP_200_OK

    @patch("netbox_rir_manager.api.views.ARINBackend")
    def test_delete_arin_api(self, mock_backend_cls, admin_api_client, rir_network, rir_user_key):
        mock_backend = MagicMock()
        mock_backend.delete_network.return_value = {
            "ticket_number": "TKT-API-DELETE",
            "ticket_type": "NET_DELETE_REQUEST",
            "ticket_status": "PENDING_REVIEW",
            "raw_data": {},
        }
        mock_backend_cls.from_rir_config.return_value = mock_backend

        url = reverse("plugins-api:netbox_rir_manager-api:rirnetwork-delete-arin", args=[rir_network.pk])
        response = admin_api_client.post(url, format="json")
        assert response.status_code == status.HTTP_201_CREATED

    def test_ticket_refresh_api(self, admin_api_client, rir_ticket):
        url = reverse("plugins-api:netbox_rir_manager-api:rirticket-refresh", args=[rir_ticket.pk])
        response = admin_api_client.post(url, format="json")
        assert response.status_code == status.HTTP_200_OK



from netbox.plugins import PluginConfig

__version__ = "0.3.1"


class NetBoxRIRManagerConfig(PluginConfig):
    name = "netbox_rir_manager"
    verbose_name = "NetBox RIR Manager"
    description = "Manage RIR (ARIN, RIPE, etc.) resources directly from NetBox"
    version = __version__
    author = "Jonathan Senecal"
    author_email = "contact@jonathansenecal.com"
    base_url = "rir-manager"
    min_version = "4.5.0"
    required_settings = []
    default_settings = {
        "top_level_menu": True,
        "sync_interval_hours": 24,
        "auto_link_networks": True,
        "enabled_backends": ["ARIN"],
        "encryption_key": "",
        "api_retry_count": 3,
        "api_retry_backoff": 2,
        "geocoding_provider": "nominatim",
        "google_geocoding_api_key": "",
    }

    def ready(self):
        super().ready()
        from . import signals  # noqa: F401


config = NetBoxRIRManagerConfig
